@@ -4,10 +4,17 @@
 /* Author: Rainer Doemer			first version: 08/31/97 */
 /************************************************************************/
 
-/* last update: 05/25/01 */
+
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 05/29/03 RD	get definitions for standard import directory from CcDriver.h
+ * 11/03/02 RD	fixed clearing of settings lists (after code review by AG)
+ * 02/13/02 RD	added standard import directory for default import search path
+ * 11/13/01 RD	added command line options for code generation parameters
+ * 11/08/01 RD	switched code generation to use GL_IO layer
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
  * 05/25/01 RD	eliminated support for automatic IP wrapping (scc -ip option)
  * 05/25/01 RD	first version adjusted for SCRC
@@ -117,6 +124,9 @@ static BOOL		DebugMode	= FALSE;
 static BOOL		OptimizeMode	= FALSE;
 static BOOL		GenerateLines	= TRUE;
 static BOOL		GenerateNotes	= TRUE;
+static int		TabStep		= GL_TAB_STEP_DEFAULT;
+static int		SysTabStep	= GL_TAB_STEP_SYSTEM;
+static int		LineWrapping	= GL_LINE_WRAP_DEFAULT;
 
 static BOOL		OptionIorOrequired = FALSE;
 static BOOL		GenerateH	= TRUE;
@@ -194,6 +204,9 @@ switch (ErrorNumber)
 	  return("Explicit input or output filename required"
 							GL_ERROR_MSG_NEWLINE
 		"(use option -i or -o to avoid overriding the input file)");
+     case SCC_ERROR_INVALID_VALUE_FOR_OPTION:
+	  return(ErrorMessage.chars()
+		/* "Invalid value (%d) for option -%s" */);
     } /* hctiws */
 
 return("Unknown error number (internal)");
@@ -234,7 +247,14 @@ fputs(
 "-w|-ww|-www      increase warning level         no warnings\n"
 "-g|-O            enable debugging/optimization  none\n"
 "-sl              suppress gen. of line infos    enabled\n"
-"-sn              suppress gen. of annotations   enabled\n"
+"-sn              suppress gen. of annotations   enabled\n", File);
+fprintf(File,
+"-st <tab step>   set tabulator stepping         %d\n", GL_TAB_STEP_DEFAULT);
+fprintf(File,
+"-sT <sys tab>    set system tabulator stepping  %d\n", GL_TAB_STEP_SYSTEM);
+fprintf(File,
+"-sw <line wrap>  set column for line wrapping   %d\n", GL_LINE_WRAP_DEFAULT);
+fputs(
 "-i <infile|->    specify input filename (%i)    <design>.<suffix>\n"
 "-o <outfile|->   specify output filename (%o)   <design>.<suffix>\n"
 "-D               do not define standard macros  " CCD_USAGE_DEFINES "\n"
@@ -247,7 +267,7 @@ fputs(
 "-L<dir>          insert dir in library path     none\n"
 "-l               do not use standard libraries  " CCD_USAGE_STD_LIBRARIES "\n"
 "-l<lib>          link against library lib       none\n"
-"-P               clear standard import path     <current dir>\n"
+"-P               clear standard import path     " CCD_SPECC_STD_IMPORT "\n"
 "-P<dir>          insert dir in import path      none\n"
 "-xpp <pp call>   override preprocessor call     " CCD_USAGE_PP_CALL "\n"
 "-xcc <cc call>   override compiler call         " CCD_USAGE_CC_CALL "\n"
@@ -269,6 +289,8 @@ static ERROR GetArguments(		/* parse command line arguments */
 {
 GL_STRINGLIST	*Succ,
 		*TmpDirList;
+static string	StdImportDir;
+const char	*TmpStr;
 ERROR		Error;
 int		Arg;
 
@@ -306,7 +328,15 @@ assert(PDirList == NULL);		/* default import path */
 assert(PDirListInsert == NULL);
 PDirList = new GL_STRINGLIST();
 PDirList->Succ = NULL;
-PDirList->String = ".";
+PDirList->String = CCD_SPECC_STD_IMPORT1;
+if ((TmpStr = getenv(CCD_SPECC_ENV_VAR)))
+   { StdImportDir = TmpStr;
+     StdImportDir += "/";
+     StdImportDir += CCD_SPECC_IMPORT_DIR;
+     PDirList->Succ = new GL_STRINGLIST();
+     PDirList->Succ->Succ = NULL;
+     PDirList->Succ->String = StdImportDir.chars();
+    } /* fi */
 
 Arg++;
 if (Arg >= argc)
@@ -393,6 +423,57 @@ while(Arg < argc)			/* process all options */
 	  Arg++;
 	  continue;
 	 } /* fi */
+     if (0 == strcmp(&argv[Arg][1], "st"))
+	{ if (++Arg >= argc)
+	     { ErrorMessage = "Option -st requires an integer argument";
+	       return(SCC_ERROR_OPTION_NEEDS_ARGUMENT);
+	      } /* fi */
+	  if (argv[Arg][0] == 0)
+	     { ErrorMessage = "Empty argument for option -st";
+	       return(SCC_ERROR_EMPTY_ARGUMENT);
+	      } /* fi */
+	  TabStep = atoi(argv[Arg++]);
+	  if (TabStep < 0)
+	     { ErrorMessage.form("Invalid value (%d) for option -st",
+							TabStep);
+	       return(SCC_ERROR_INVALID_VALUE_FOR_OPTION);
+	      } /* fi */
+	  continue;
+	 } /* fi */
+     if (0 == strcmp(&argv[Arg][1], "sT"))
+	{ if (++Arg >= argc)
+	     { ErrorMessage = "Option -sT requires an integer argument";
+	       return(SCC_ERROR_OPTION_NEEDS_ARGUMENT);
+	      } /* fi */
+	  if (argv[Arg][0] == 0)
+	     { ErrorMessage = "Empty argument for option -sT";
+	       return(SCC_ERROR_EMPTY_ARGUMENT);
+	      } /* fi */
+	  SysTabStep = atoi(argv[Arg++]);
+	  if (SysTabStep < 0)
+	     { ErrorMessage.form("Invalid value (%d) for option -sT",
+							SysTabStep);
+	       return(SCC_ERROR_INVALID_VALUE_FOR_OPTION);
+	      } /* fi */
+	  continue;
+	 } /* fi */
+     if (0 == strcmp(&argv[Arg][1], "sw"))
+	{ if (++Arg >= argc)
+	     { ErrorMessage = "Option -sw requires an integer argument";
+	       return(SCC_ERROR_OPTION_NEEDS_ARGUMENT);
+	      } /* fi */
+	  if (argv[Arg][0] == 0)
+	     { ErrorMessage = "Empty argument for option -sw";
+	       return(SCC_ERROR_EMPTY_ARGUMENT);
+	      } /* fi */
+	  LineWrapping = atoi(argv[Arg++]);
+	  if (LineWrapping < 0)
+	     { ErrorMessage.form("Invalid value (%d) for option -sw",
+							LineWrapping);
+	       return(SCC_ERROR_INVALID_VALUE_FOR_OPTION);
+	      } /* fi */
+	  continue;
+	 } /* fi */
      if (0 == strcmp(&argv[Arg][1], "i"))
 	{ if (++Arg >= argc)
 	     { ErrorMessage = "Option -i requires an argument";
@@ -434,7 +515,14 @@ while(Arg < argc)			/* process all options */
 	 } /* fi */
      if (argv[Arg][1] == 'D')
 	{ if (argv[Arg][2] == 0)
-	     { if ((SCC_Error = CCD_NoStdMacroDefs()))
+	     { while(DDefList)
+		  { Succ = DDefList->Succ;
+		    delete DDefList;
+		    DDefList = Succ;
+		   } /* eliwh */
+	       DDefList = NULL;	/* empty definition list */
+	       DDefListLast = NULL;
+	       if ((SCC_Error = CCD_NoStdMacroDefs()))
 		  { return(SCC_Error);
 		   } /* fi */
 	      } /* fi */
@@ -456,7 +544,14 @@ while(Arg < argc)			/* process all options */
 	 } /* fi */
      if (argv[Arg][1] == 'U')
 	{ if (argv[Arg][2] == 0)
-	     { if ((SCC_Error = CCD_NoStdUnDefs()))
+	     { while(UDefList)
+		  { Succ = UDefList->Succ;
+		    delete UDefList;
+		    UDefList = Succ;
+		   } /* eliwh */
+	       UDefList = NULL;	/* empty un-definition list */
+	       UDefListLast = NULL;
+	       if ((SCC_Error = CCD_NoStdUnDefs()))
 		  { return(SCC_Error);
 		   } /* fi */
 	      } /* fi */
@@ -478,7 +573,14 @@ while(Arg < argc)			/* process all options */
 	 } /* fi */
      if (argv[Arg][1] == 'I')
 	{ if (argv[Arg][2] == 0)
-	     { if ((SCC_Error = CCD_NoStdIncludePath()))
+	     { while(IDirList)
+		  { Succ = IDirList->Succ;
+		    delete IDirList;
+		    IDirList = Succ;
+		   } /* eliwh */
+	       IDirList = NULL;	/* empty include directory list */
+	       IDirListLast = NULL;
+	       if ((SCC_Error = CCD_NoStdIncludePath()))
 		  { return(SCC_Error);
 		   } /* fi */
 	      } /* fi */
@@ -500,7 +602,14 @@ while(Arg < argc)			/* process all options */
 	 } /* fi */
      if (argv[Arg][1] == 'L')
 	{ if (argv[Arg][2] == 0)
-	     { if ((SCC_Error = CCD_NoStdLibraryPath()))
+	     { while(LDirList)
+		  { Succ = LDirList->Succ;
+		    delete LDirList;
+		    LDirList = Succ;
+		   } /* eliwh */
+	       LDirList = NULL;	/* empty library directory list */
+	       LDirListLast = NULL;
+	       if ((SCC_Error = CCD_NoStdLibraryPath()))
 		  { return(SCC_Error);
 		   } /* fi */
 	      } /* fi */
@@ -522,7 +631,14 @@ while(Arg < argc)			/* process all options */
 	 } /* fi */
      if (argv[Arg][1] == 'l')
 	{ if (argv[Arg][2] == 0)
-	     { if ((SCC_Error = CCD_NoStdLibraries()))
+	     { while(lLibList)
+		  { Succ = lLibList->Succ;
+		    delete lLibList;
+		    lLibList = Succ;
+		   } /* eliwh */
+	       lLibList = NULL;	/* empty libraries list */
+	       lLibListLast = NULL;
+	       if ((SCC_Error = CCD_NoStdLibraries()))
 		  { return(SCC_Error);
 		   } /* fi */
 	      } /* fi */
@@ -936,8 +1052,8 @@ int main(int argc, char **argv)
 {
 int		i;
 ERROR		Error;
-FILE		*InFile,
-		*OutFile;
+FILE		*InFile;
+GL_HANDLE	*IO;
 #ifdef SCC_DEBUG
 GL_STRINGLIST	*StringPtr;
 #endif /* SCC_DEBUG */
@@ -996,6 +1112,9 @@ fprintf(stderr, "SCC_DEBUG: DebugMode      = %d\n", DebugMode);
 fprintf(stderr, "SCC_DEBUG: OptimizeMode   = %d\n", OptimizeMode);
 fprintf(stderr, "SCC_DEBUG: GenerateLines  = %d\n", GenerateLines);
 fprintf(stderr, "SCC_DEBUG: GenerateNotes  = %d\n", GenerateNotes);
+fprintf(stderr, "SCC_DEBUG: TabStep        = %d\n", TabStep);
+fprintf(stderr, "SCC_DEBUG: SysTabStep     = %d\n", SysTabStep);
+fprintf(stderr, "SCC_DEBUG: LineWrapping   = %d\n", LineWrapping);
 fprintf(stderr, "SCC_DEBUG: GenerateH      = %d\n", GenerateH);
 fprintf(stderr, "SCC_DEBUG: GenerateCC     = %d\n", GenerateCC);
 fprintf(stderr, "SCC_DEBUG: ImportPath     = ");
@@ -1122,23 +1241,29 @@ if (CallTranslator)
 	{ if (VerbosityLevel >= 2)
 	     { fprintf(stderr, "   Output: \"%s\"\n", FilenameH.chars());
 	      } /* fi */
-	  if (!(OutFile = GL_OpenFileForWriting(FilenameH.chars())))
+	  if (!(IO = GL_IO::OpenFile(FilenameH.chars(), GL_IO_WRITE)))
 	     { GL_PrintError(GL_Error, GL_ErrorText(GL_Error));
 	       fputs("Aborted.\n", stderr);
 	       CleanUp();
 	       return(RESULT_ERROR);
 	      } /* fi */
+	  IO->SetTabStep(TabStep);
+	  assert(GL_Error == GL_ERROR_NO_ERROR);
+	  IO->SetSystemTabStep(SysTabStep);
+	  assert(GL_Error == GL_ERROR_NO_ERROR);
+	  IO->SetLineWrap(LineWrapping);
+	  assert(GL_Error == GL_ERROR_NO_ERROR);
 	  assert(TheDesign != NULL);
-	  if ((Error = TheDesign->WriteH(OutFile,
+	  if ((Error = TheDesign->WriteH(IO,
 			FilenameH.chars(),
 			GenerateNotes, GenerateLines)))
 	     { GL_PrintError(Error, SIR_ErrorText(Error));
 	       fputs("Aborted.\n", stderr);
-	       GL_CloseFile(OutFile);
+	       IO->Close();
 	       CleanUp();
 	       return(RESULT_ERROR);
 	      } /* fi */
-	  if ((Error = GL_CloseFileWithCheck(OutFile)))
+	  if ((Error = IO->CloseWithCheck()))
 	     { GL_PrintError(Error, GL_ErrorText(Error));
 	       fputs("Aborted.\n", stderr);
 	       CleanUp();
@@ -1149,23 +1274,29 @@ if (CallTranslator)
 	{ if (VerbosityLevel >= 2)
 	     { fprintf(stderr, "   Output: \"%s\"\n", FilenameCC.chars());
 	      } /* fi */
-	  if (!(OutFile = GL_OpenFileForWriting(FilenameCC.chars())))
+	  if (!(IO = GL_IO::OpenFile(FilenameCC.chars(), GL_IO_WRITE)))
 	     { GL_PrintError(GL_Error, GL_ErrorText(GL_Error));
 	       fputs("Aborted.\n", stderr);
 	       CleanUp();
 	       return(RESULT_ERROR);
 	      } /* fi */
+	  IO->SetTabStep(TabStep);
+	  assert(GL_Error == GL_ERROR_NO_ERROR);
+	  IO->SetSystemTabStep(SysTabStep);
+	  assert(GL_Error == GL_ERROR_NO_ERROR);
+	  IO->SetLineWrap(LineWrapping);
+	  assert(GL_Error == GL_ERROR_NO_ERROR);
 	  assert(TheDesign != NULL);
-	  if ((Error = TheDesign->WriteCC(OutFile,
+	  if ((Error = TheDesign->WriteCC(IO,
 			FilenameCC.chars(), FilenameH.chars(),
 			GenerateNotes, GenerateLines)))
 	     { GL_PrintError(Error, SIR_ErrorText(Error));
 	       fputs("Aborted.\n", stderr);
-	       GL_CloseFile(OutFile);
+	       IO->Close();
 	       CleanUp();
 	       return(RESULT_ERROR);
 	      } /* fi */
-	  if ((Error = GL_CloseFileWithCheck(OutFile)))
+	  if ((Error = IO->CloseWithCheck()))
 	     { GL_PrintError(Error, GL_ErrorText(Error));
 	       fputs("Aborted.\n", stderr);
 	       CleanUp();
@@ -1220,23 +1351,29 @@ if (CallDeparser)
 	{ fprintf(stderr, "   Input:  <internal representation>\n");
 	  fprintf(stderr, "   Output: \"%s\"\n", FilenameSCout.chars());
 	 } /* fi */
-     if (!(OutFile = GL_OpenFileForWriting(FilenameSCout.chars())))
+     if (!(IO = GL_IO::OpenFile(FilenameSCout.chars(), GL_IO_WRITE)))
 	{ GL_PrintError(GL_Error, GL_ErrorText(GL_Error));
 	  fputs("Aborted.\n", stderr);
 	  CleanUp();
 	  return(RESULT_ERROR);
 	 } /* fi */
+     IO->SetTabStep(TabStep);
+     assert(GL_Error == GL_ERROR_NO_ERROR);
+     IO->SetSystemTabStep(SysTabStep);
+     assert(GL_Error == GL_ERROR_NO_ERROR);
+     IO->SetLineWrap(LineWrapping);
+     assert(GL_Error == GL_ERROR_NO_ERROR);
      assert(TheDesign != NULL);
-     if ((Error = TheDesign->WriteSC(OutFile,
+     if ((Error = TheDesign->WriteSC(IO,
 			FilenameSCout.chars(),
 			GenerateNotes, GenerateLines)))
 	{ GL_PrintError(Error, SIR_ErrorText(Error));
 	  fputs("Aborted.\n", stderr);
-	  GL_CloseFile(OutFile);
+	  IO->Close();
 	  CleanUp();
 	  return(RESULT_ERROR);
 	 } /* fi */
-     if ((Error = GL_CloseFileWithCheck(OutFile)))
+     if ((Error = IO->CloseWithCheck()))
 	{ GL_PrintError(Error, GL_ErrorText(Error));
 	  fputs("Aborted.\n", stderr);
 	  CleanUp();
