@@ -4,10 +4,13 @@
 /* Author: Rainer Doemer			first version: 11/02/01 */
 /************************************************************************/
 
-/* last update: 11/21/01 */
+/* last update: 06/21/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/21/04 PC 	Introduced Macro EOL 
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 01/15/02 RD	bug fix: clear the IO string before writing into it
  * 11/21/01 RD	took out default arguments from function definitions
  * 11/13/01 RD	added support for line wrapping
  * 11/08/01 RD	completed implementation of class GL_IO
@@ -191,6 +194,10 @@ GL_HANDLE *GL_IO::OpenString(	/* open a string */
 
 assert(String != NULL);
 
+if (Mode == GL_IO_WRITE)
+   { *String = "";	/* clear the string */
+    } /* fi */
+
 return(new GL_IO(Mode, String));
 
 } /* end of GL_IO::OpenString */
@@ -836,17 +843,18 @@ if (  (Mode != GL_IO_WRITE)
      return;
     } /* fi */
 
+//PC 06/21/04
 if (FileHandle)
-   { fputc('\n', FileHandle);
+   { fputs(EOL, FileHandle);
     } /* fi */
 else
    { assert(StringHandle != NULL);
      if (Pos + 1 <= StringHandle->length())
-	{ StringHandle->at((int)Pos, 1) = '\n';
+	{ StringHandle->at((int)Pos, 1) = EOL;
 	 } /* fi */
      else
 	{ if (Pos == StringHandle->length())
-	     { *StringHandle += '\n';
+	     { *StringHandle += EOL;
 	      } /* fi */
 	  else
 	     { GL_Error = GL_ERROR_ILLEGAL_ARGUMENTS;
