@@ -4,10 +4,12 @@
 /* Author: Andreas Gerstlauer <gerstl>		first version: 03/02/00 */
 /************************************************************************/
 
-/* last update: 11/21/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 04/19/02 AG	fixed a bug in string comparing, added regression test
  * 11/21/01 RD	took out default arguments from function definitions
  * 11/21/01 RD	started this header (last change was 11/05/01 by AG)
  */
@@ -289,12 +291,16 @@ SubString& SubString::operator =  (char        c)
 
 int compare(const String& x, const String& y) 
 {
-  return strncmp(x.chars(), y.chars(), MIN(x.length(), y.length()));
+  return strncmp(x.chars(), y.chars(), MAX(x.length(), y.length()));
 }
 
 int compare(const String& x, const char* t) 
 {
-  return strncmp(x.chars(), t, x.length());
+  int res = strncmp(x.chars(), t, x.length());
+  if (!res) {
+    return x.length() - strlen(t);
+  }
+  return res;
 }
 
 int compare(const SubString& x, const String& y) 
@@ -348,7 +354,7 @@ int compare(const SubString& x, const char* t)
 
 #ifdef DEBUG	/* module self test */
 
-void String_Error(String& S, const char* msg)
+void String_Error(String S, const char* msg)
 {
    fprintf(stderr, "String error: %s is %s\n", msg, (const char*)S );
    exit(1);
@@ -517,7 +523,11 @@ int main(int argc, char **argv)
 
    
    /* Some regression test */
-   
+
+   String s1("ab");
+   String s2("abc");
+   if( s1 == s2 ) String_Error("TRUE", "'ab' == 'abc'");
+  
    String pes("proc1 proc2");
    pes += ' ';   
    pes = pes.after(' ');
