@@ -4,10 +4,11 @@
 /* Author: Rainer Doemer			first version: 10/10/97 */
 /************************************************************************/
 
-/* last update: 11/08/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
  * 11/08/01 RD	removed SIR_ERROR_WRITING_SPECC_FILE
  * 05/31/01 RD	removed code not needed for the SCRC
  * 02/14/01 RD	started this list (last changes was 04/08/99)
@@ -211,6 +212,11 @@ switch (ErrorNumber)
      case SIR_ERROR_REDEFINITION_OF_INTERFACE_2:
 	  return(SIR_ErrMsg.chars()
 	/* "Redefinition of %s '%s'" */);
+     case SIR_ERROR_ENUM_DEFINED_TWICE:
+          return(SIR_ErrMsg.chars()
+        /* "Error while importing:" GL_ERROR_MSG_NEWLINE
+                "Enum constant '%s' defined twice with"
+                " different values (%d, %d)" */);
      case SIR_ERROR_DOUBLE_IMP_SYMBOL_NOT_FOUND:
 	  return("Error while importing:" GL_ERROR_MSG_NEWLINE
 		"double-imported symbol not found (internal)");
@@ -237,6 +243,9 @@ switch (ErrorNumber)
      case SIR_ERROR_INSTANCE_CYCLIC_NESTING:
 	  return(SIR_ErrMsg.chars()
 	/* "Cyclic dependency in class instantiation '%s' (internal)" */);
+     case SIR_ERROR_VARIABLE_CYCLIC_NESTING:
+          return(SIR_ErrMsg.chars()
+        /* "Cyclic dependency in variable definition '%s' (internal)" */);
      case SIR_ERROR_CLASS_CYCLIC_NESTING:
 	  return(SIR_ErrMsg.chars()
 	/* "Cyclic nesting in class definition '%s' (internal)" */);
@@ -261,10 +270,6 @@ switch (ErrorNumber)
 	/* "Behavioral hierarchy not clean in compound statement;"
 							GL_ERROR_MSG_NEWLINE
 		"unable to generate simulation model" */);
-     case SIR_ERROR_COMPLEX_MEMBER_INITIALIZER:
-	  return(SIR_ErrMsg.chars()
-	/* "Complex initializer not supported" GL_ERROR_MSG_NEWLINE
-		"for class member '%s'" */);
      case SIR_ERROR_ILLEGAL_INTERFACE_MEMBER:
 	  return(SIR_ErrMsg.chars()
 	/* "Interface member '%s' is not a function declaration" */);
@@ -507,6 +512,126 @@ switch (ErrorNumber)
      case SIR_ERROR_PORT_READ_ACCESS:
 	  return(SIR_ErrMsg.chars()
 	/* "Read access to output port%s" */);
+     case SIR_ERROR_REDEFINITION_OF_STATE:
+          return(SIR_ErrMsg.chars()
+        /* "Redefinition of local FSM state '%s'" */);
+     case SIR_ERROR_TYPE_BUFFERED_SUBTYPE:
+          return("Invalid type: 'buffered' or 'signal' subtype");
+     case SIR_ERROR_TYPE_FUNCTION_SUBTYPE:
+          return("Invalid type: function subtype");
+     case SIR_ERROR_BUFFERED_LOCAL_USERTYPE:
+          return("'signal' or 'buffered' type not supported"
+                                                GL_ERROR_MSG_NEWLINE
+                        "for local user-defined types (struct/union/enum)");
+     case SIR_ERROR_ILLEGAL_BUFFER_CLOCK_SPEC:
+          return(SIR_ErrMsg.chars()
+        /* "Storage class '%s' forbids definition" GL_ERROR_MSG_NEWLINE
+                        "of clock specifier for buffered variable '%s'" */);
+     case SIR_ERROR_MISSING_BUFFER_CLOCK_SPEC:
+          return(SIR_ErrMsg.chars()
+        /* "Clock specifier missing for buffered variable '%s'" */);
+     case SIR_ERROR_LVALUE_REQUIRED_FOR_ADDR_OP:
+          return("Lvalue required as argument for unary '&' operator");
+     case SIR_ERROR_INVALID_ARGUMENT_FOR_ADDR_OP:
+          return("Invalid argument for unary '&' operator");
+     case SIR_ERROR_INVALID_OPERAND_FOR_COMP_OP:
+          return("Invalid operand for comparison operator");
+     case SIR_ERROR_INVALID_TYPE_FOR_ASSIGN_OP:
+          return("Invalid type for assignment operator");
+     case SIR_ERROR_INVALID_TYPE_CAST:
+          return("Invalid type cast");
+     case SIR_ERROR_ILLEGAL_SIZEOF_SIGNAL:
+          return("Illegal 'signal' type for 'sizeof' operator");
+     case SIR_ERROR_ILLEGAL_SIZEOF_BUFFER:
+          return("Illegal 'buffered' type for 'sizeof' operator");
+     case SIR_ERROR_PIPED_SIGNAL_OR_BUFFER:
+          return("Invalid storage class 'piped' for 'signal'"
+                                                " or 'buffered' variable");
+     case SIR_ERROR_PIPED_ARRAY_OF_UNKNOWN_SIZE:
+          return("Invalid 'piped' array with unspecified size");
+     case SIR_ERROR_EVENT_EXPRESSION:
+          return(SIR_ErrMsg.chars()
+        /* "Event '%s' must not be used in any expression" */);
+     case SIR_ERROR_PORT_MAPPING_MISMATCH_11:
+          return(SIR_ErrMsg.chars()
+        /* "Mismatch in port mapping for port %u:" GL_ERROR_MSG_NEWLINE
+                "%s type expected" */);
+     case SIR_ERROR_QUALIFIED_BUFFERED_TYPE:
+          return("'buffered' or 'signal' type must not be qualified"
+                                                        GL_ERROR_MSG_NEWLINE
+                "as 'const' or 'volatile'");
+     case SIR_ERROR_QUALIFIED_PIPED_TYPE:
+          return("'piped' storage class must not be used with type"
+                                                        GL_ERROR_MSG_NEWLINE
+                "qualified as 'const' or 'volatile'");
+     case SIR_ERROR_GOTO_LABEL_IN_FSMD:
+          return(SIR_ErrMsg.chars()
+        /* "Target label '%s' for 'goto' defined inside 'fsmd'" */);
+     case SIR_ERROR_REDEFINITION_OF_FSMD_STATE:
+          return(SIR_ErrMsg.chars()
+        /* "Redefinition of fsmd state '%s'" */);
+     case SIR_ERROR_ARRAY_SIZE_REQUIRED:
+          return("Invalid type; array size required");
+     case SIR_ERROR_SYMBOL_NOT_VISIBLE_IN_SCOPE:
+          return(SIR_ErrMsg.chars()
+        /* "symbol %s is not visible in scope" */);
+     case SIR_ERROR_USERTYPE_NOT_VISIBLE_IN_SCOPE:
+          return(SIR_ErrMsg.chars()
+        /* "type %s %s is not visible in scope" */);
+     case SIR_ERROR_INVALID_DELAYED_ASSIGNMENT_1:
+          return(SIR_ErrMsg.chars()
+        /* "Condition %s" GL_ERROR_MSG_NEWLINE
+                "of '%s' assignment in state %s" GL_ERROR_MSG_NEWLINE
+                "(%s)" GL_ERROR_MSG_NEWLINE
+                "cannot be evaluated in state %s" GL_ERROR_MSG_NEWLINE
+                "(%s)" */);
+     case SIR_ERROR_INVALID_DELAYED_ASSIGNMENT_2:
+          return(SIR_ErrMsg.chars()
+        /* "Target expression %s" GL_ERROR_MSG_NEWLINE
+                "of '%s' assignment in state %s" GL_ERROR_MSG_NEWLINE
+                "(%s)" GL_ERROR_MSG_NEWLINE
+                "cannot be evaluated in state %s" GL_ERROR_MSG_NEWLINE
+                "(%s)" */);
+     case SIR_ERROR_BREAK_OUTSIDE_LOOP_OR_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "'break' statement outside loop or 'switch' statement%s" */);
+     case SIR_ERROR_CONTINUE_OUTSIDE_LOOP:
+          return(SIR_ErrMsg.chars()
+        /* "'continue' statement outside loop statement%s" */);
+     case SIR_ERROR_CASE_OUTSIDE_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "'case' statement outside 'switch' statement%s" */);
+     case SIR_ERROR_DEFAULT_OUTSIDE_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "'default' statement outside 'switch' statement%s" */);
+     case SIR_ERROR_CASE_NOT_2_BELOW_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "'case' statement not in compound statement"
+                                " below 'switch' statement%s" */);
+     case SIR_ERROR_DFLT_NOT_2_BELOW_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "'default' statement not in compound statement"
+                                " below 'switch' statement%s" */);
+     case SIR_ERROR_BREAK_NOT_4_BELOW_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "'break' statement not at fourth level"
+                                " below 'switch' statement%s" */);
+     case SIR_ERROR_BREAK_NOT_LAST_IN_CASE:
+          return(SIR_ErrMsg.chars()
+        /* "'break' statement not last statement in case%s" */);
+     case SIR_ERROR_SWITCH_BODY_NOT_COMPOUND:
+          return(SIR_ErrMsg.chars()
+        /* "'switch' statement body not a compound statement%s" */);
+     case SIR_ERROR_NOT_CASE_DEFAULT_IN_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "only 'case' and 'default' statements allowed in 'switch' body%s" */)
+;
+     case SIR_ERROR_CASE_NOT_ENDING_WITH_BREAK:
+          return(SIR_ErrMsg.chars()
+        /* "case not a compound statement ending with 'break'%s" */);
+     case SIR_ERROR_MULTIPLE_DEFAULT_IN_SWITCH:
+          return(SIR_ErrMsg.chars()
+        /* "multiple 'default' statements in 'switch' body%s" */);
     } /* hctiws */
 
 return("Unknown error number (internal)");
