@@ -4,10 +4,16 @@
 /* Author: Rainer Doemer			first version: 09/24/97 */
 /************************************************************************/
 
-/* last update: 11/08/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 12/04/03 HY	added support for HAVE_METHOD
+ * 10/28/03 RD	adjusted for new member ParentStmnt in SIR_Statement
+ * 02/15/02 RD	eliminated sizeof() difference between SpecC and C++
+ *		(otherwise bitvectors cannot be malloc()ed)
+ * 01/15/02 RD	added support for composite annotations
  * 11/08/01 RD	switched code generation to use GL_IO layer
  * 05/29/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
@@ -375,9 +381,9 @@ printf("\t(0000111b*0000110b)^(1000ll-958) = %d\t\t%s\n",
 					"Correct." : (Faults++, "WRONG!!!")));
 delete MyExpression;
 
-Result1 = TypeBool->SizeOf(&Alignment, FALSE);
+Result1 = TypeBool->SizeOf(&Alignment);
 TypeArray10->SubType = TypeBool;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(bool)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(bool))
@@ -385,9 +391,9 @@ printf("\tsizeof(bool)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeChar->SizeOf(&Alignment, FALSE);
+Result1 = TypeChar->SizeOf(&Alignment);
 TypeArray10->SubType = TypeChar;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(char)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(char))
@@ -395,9 +401,9 @@ printf("\tsizeof(char)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeShort->SizeOf(&Alignment, FALSE);
+Result1 = TypeShort->SizeOf(&Alignment);
 TypeArray10->SubType = TypeShort;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(short)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(short))
@@ -405,9 +411,9 @@ printf("\tsizeof(short)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeInt->SizeOf(&Alignment, FALSE);
+Result1 = TypeInt->SizeOf(&Alignment);
 TypeArray10->SubType = TypeInt;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(int)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(int))
@@ -415,9 +421,9 @@ printf("\tsizeof(int)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeLong->SizeOf(&Alignment, FALSE);
+Result1 = TypeLong->SizeOf(&Alignment);
 TypeArray10->SubType = TypeLong;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(long)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(long))
@@ -425,9 +431,9 @@ printf("\tsizeof(long)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeLongLong->SizeOf(&Alignment, FALSE);
+Result1 = TypeLongLong->SizeOf(&Alignment);
 TypeArray10->SubType = TypeLongLong;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(lg.long)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(LONG_LONG))
@@ -435,9 +441,9 @@ printf("\tsizeof(lg.long)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeFloat->SizeOf(&Alignment, FALSE);
+Result1 = TypeFloat->SizeOf(&Alignment);
 TypeArray10->SubType = TypeFloat;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(float)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(float))
@@ -445,9 +451,9 @@ printf("\tsizeof(float)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeDouble->SizeOf(&Alignment, FALSE);
+Result1 = TypeDouble->SizeOf(&Alignment);
 TypeArray10->SubType = TypeDouble;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(double)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(double))
@@ -455,9 +461,9 @@ printf("\tsizeof(double)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeLongDouble->SizeOf(&Alignment, FALSE);
+Result1 = TypeLongDouble->SizeOf(&Alignment);
 TypeArray10->SubType = TypeLongDouble;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(lg.dbl.)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(long double))
@@ -466,9 +472,9 @@ printf("\tsizeof(lg.dbl.)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 			? "Correct." : (Faults++, "WRONG!!!")));
 
 #ifdef INCLUDE_TEMPLATES
-Result1 = TypeBitVec4->SizeOf(&Alignment, FALSE);
+Result1 = TypeBitVec4->SizeOf(&Alignment);
 TypeArray10->SubType = TypeBitVec4;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 #ifdef WORK_AROUND_EGCS112_BUGS
 SizeOfTmp1 = sizeof(bit<4>);
 SizeOfTmp2 = sizeof(bit<4>[10]);
@@ -487,9 +493,9 @@ printf("\tsizeof(bit[4])\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 			? "Correct." : (Faults++, "WRONG!!!")));
 #endif /* WORK_AROUND_EGCS112_BUGS */
 
-Result1 = TypeBitVec64->SizeOf(&Alignment, FALSE);
+Result1 = TypeBitVec64->SizeOf(&Alignment);
 TypeArray10->SubType = TypeBitVec64;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(bit[64])\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(bit<64>))
@@ -498,9 +504,9 @@ printf("\tsizeof(bit[64])\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 			? "Correct." : (Faults++, "WRONG!!!")));
 #endif /* INCLUDE_TEMPLATES */
 
-Result1 = TypeVoidPtr->SizeOf(&Alignment, FALSE);
+Result1 = TypeVoidPtr->SizeOf(&Alignment);
 TypeArray10->SubType = TypeVoidPtr;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(void*)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(void*))
@@ -508,9 +514,9 @@ printf("\tsizeof(void*)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeE1->SizeOf(&Alignment, FALSE);
+Result1 = TypeE1->SizeOf(&Alignment);
 TypeArray10->SubType = TypeE1;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(e1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(e1))
@@ -518,9 +524,9 @@ printf("\tsizeof(e1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeS1->SizeOf(&Alignment, FALSE);
+Result1 = TypeS1->SizeOf(&Alignment);
 TypeArray10->SubType = TypeS1;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(s1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(s1))
@@ -528,9 +534,9 @@ printf("\tsizeof(s1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeS2->SizeOf(&Alignment, FALSE);
+Result1 = TypeS2->SizeOf(&Alignment);
 TypeArray10->SubType = TypeS2;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(s2)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(s2))
@@ -538,9 +544,9 @@ printf("\tsizeof(s2)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeS3->SizeOf(&Alignment, FALSE);
+Result1 = TypeS3->SizeOf(&Alignment);
 TypeArray10->SubType = TypeS3;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(s3)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(s3))
@@ -548,9 +554,9 @@ printf("\tsizeof(s3)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeS4->SizeOf(&Alignment, FALSE);
+Result1 = TypeS4->SizeOf(&Alignment);
 TypeArray10->SubType = TypeS4;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(s4)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(s4))
@@ -558,9 +564,9 @@ printf("\tsizeof(s4)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeB1->SizeOf(&Alignment, FALSE);
+Result1 = TypeB1->SizeOf(&Alignment);
 TypeArray10->SubType = TypeB1;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(b1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(b1))
@@ -568,9 +574,9 @@ printf("\tsizeof(b1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeB2->SizeOf(&Alignment, FALSE);
+Result1 = TypeB2->SizeOf(&Alignment);
 TypeArray10->SubType = TypeB2;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(b2)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(b2))
@@ -578,9 +584,9 @@ printf("\tsizeof(b2)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeB3->SizeOf(&Alignment, FALSE);
+Result1 = TypeB3->SizeOf(&Alignment);
 TypeArray10->SubType = TypeB3;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(b3)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(b3))
@@ -588,9 +594,9 @@ printf("\tsizeof(b3)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeU1->SizeOf(&Alignment, FALSE);
+Result1 = TypeU1->SizeOf(&Alignment);
 TypeArray10->SubType = TypeU1;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(u1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(u1))
@@ -598,9 +604,9 @@ printf("\tsizeof(u1)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeU2->SizeOf(&Alignment, FALSE);
+Result1 = TypeU2->SizeOf(&Alignment);
 TypeArray10->SubType = TypeU2;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(u2)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(u2))
@@ -608,9 +614,9 @@ printf("\tsizeof(u2)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	  && (Alignment == Alignment2))
 			? "Correct." : (Faults++, "WRONG!!!")));
 
-Result1 = TypeU3->SizeOf(&Alignment, FALSE);
+Result1 = TypeU3->SizeOf(&Alignment);
 TypeArray10->SubType = TypeU3;
-Result2 = TypeArray10->SizeOf(&Alignment2, FALSE);
+Result2 = TypeArray10->SizeOf(&Alignment2);
 printf("\tsizeof(u3)\t= %d\t(%d-byte aligned) [%d]\t%s\n",
 	Result1, Alignment, Result2,
 	((  (Result1 == sizeof(u3))
@@ -743,9 +749,11 @@ MyFile = Design1->FileList->FindOrInsert("HelloWorld.sc");
 	/* create the note in line 1 */
 
 MyNote = new SIR_Note("FileName",	/* create the note */
-			new SIR_Constant(SIR_CONST_CHARSTRING,
-					"HelloWorld.sc",
-					1, MyFile));
+			new SIR_Initializer(
+				NULL,
+				new SIR_Constant(SIR_CONST_CHARSTRING,
+						"HelloWorld.sc",
+						1, MyFile)));
 if ((Error = Design1->Annotate(MyNote)))
    { GL_PrintError(Error, SIR_ErrorText(Error));
      puts("Building design failed, test aborted!");
@@ -816,11 +824,13 @@ Arg1Type = Design1->Types->FindOrInsert(/* argument type is 'void' */
 			SIR_TYPE_VOID);
 ArgTypes = new sir_type_ptrs(		/* build parameter list */
 		new sir_type_ptr(Arg1Type));
-MyType = Design1->Types->FindOrInsert(	/* finally build the function type */
-		SIR_TYPE_FUNCTION,
+MyType = Design1->Types->FindOrInsert(	/* finally build the method type */
+		SIR_TYPE_METHOD,
 		Design1->Types->FindOrInsert(	/* return type 'int' */
 			SIR_TYPE_INT),
-		ArgTypes);
+		ArgTypes,
+		0, FALSE, FALSE, SIR_PORT_NONE,
+		MySymbol);
 MyScope = new SIR_Symbols(		/* create a scope for parameters */
 		LocalScope,		/* (parent is behavior scope) */
 		SIR_SCOPE_PARAMETER);
@@ -902,7 +912,8 @@ MyStatement = new SIR_Statement(	/* create the labeled statement */
 		MyLabel,
 		MyStatement,	/* sub-statement */
 		11, MyFile);
-MyLabel->Statement = MyStatement;	/* fill in the statement */
+MyLabel->Statement = MyStatement;		/* fill in the statement */
+MyStatement->ParentStmnt = MyFunction->FctBody;	/* fill in the parent stmnt. */
 StatementList->Append(MyStatement);	/* append to the list of statements */
 
 	/* create the function call in line 12 */
@@ -934,6 +945,7 @@ MyStatement = new SIR_Statement(	/* create the assignment statement */
 		MyExpression,
 		NULL, NULL,	/* no sub-statements */
 		12, MyFile);
+MyStatement->ParentStmnt = MyFunction->FctBody;	/* fill in the parent stmnt. */
 StatementList->Append(MyStatement);	/* append to the list of statements */
 
 	/* create the return statement in line 13 */
@@ -955,6 +967,7 @@ MyStatement = new SIR_Statement(	/* create the return statement */
 			13, MyFile),
 		NULL, NULL,	/* no sub-statements */
 		13, MyFile);
+MyStatement->ParentStmnt = MyFunction->FctBody;	/* fill in the parent stmnt. */
 StatementList->Append(MyStatement);	/* append to the list of statements */
 
 puts(" Done.");
