@@ -4,10 +4,11 @@
 /* Author: Rainer Doemer			first version: 10/10/97 */
 /************************************************************************/
 
-/* last update: 12/28/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
  * 11/08/01 RD	removed SIR_ERROR_WRITING_SPECC_FILE
  * 05/31/01 RD	removed code not needed for the SCRC
  * 02/14/01 RD	started this list (last changes was 04/08/99)
@@ -80,6 +81,7 @@
 #define SIR_ERROR_REDEFINITION_OF_CHANNEL_2	(SIR_ERROR_BASE +  55)
 #define SIR_ERROR_REDEFINITION_OF_INTERFACE_2	(SIR_ERROR_BASE +  56)
 //      SIR_ERROR_WRITING_SPECC_FILE		(SIR_ERROR_BASE +  57)
+#define SIR_ERROR_ENUM_DEFINED_TWICE            (SIR_ERROR_BASE +  57)
 #define SIR_ERROR_DOUBLE_IMP_SYMBOL_NOT_FOUND	(SIR_ERROR_BASE +  58)
 #define SIR_ERROR_DOUBLE_IMP_USRTP_NOT_FOUND	(SIR_ERROR_BASE +  59)
 #define SIR_ERROR_DOUBLE_IMP_USRTP_NOT_FOUND_2	(SIR_ERROR_BASE +  60)
@@ -88,14 +90,14 @@
 #define SIR_ERROR_USERTYPE_DEF_IN_PARAMETERS	(SIR_ERROR_BASE +  63)
 #define SIR_ERROR_USERTYPE_CYCLIC_NESTING	(SIR_ERROR_BASE +  64)
 #define SIR_ERROR_INSTANCE_CYCLIC_NESTING	(SIR_ERROR_BASE +  65)
-#define SIR_ERROR_CLASS_CYCLIC_NESTING		(SIR_ERROR_BASE +  66)
-#define SIR_ERROR_NO_MAIN_METHOD		(SIR_ERROR_BASE +  67)
-#define SIR_ERROR_MEMBER_MAIN_NOT_A_METHOD	(SIR_ERROR_BASE +  68)
-#define SIR_ERROR_MAIN_METHOD_TYPE_MISMATCH	(SIR_ERROR_BASE +  69)
-#define SIR_ERROR_NO_PORTS_ALLOWED_FOR_MAIN	(SIR_ERROR_BASE +  70)
-#define SIR_ERROR_NO_INTERFACES_ALLOWED_FR_MAIN	(SIR_ERROR_BASE +  71)
-#define SIR_ERROR_COMPOUND_STMNT_NOT_CLEAN	(SIR_ERROR_BASE +  72)
-#define SIR_ERROR_COMPLEX_MEMBER_INITIALIZER	(SIR_ERROR_BASE +  73)
+#define SIR_ERROR_VARIABLE_CYCLIC_NESTING       (SIR_ERROR_BASE +  66)
+#define SIR_ERROR_CLASS_CYCLIC_NESTING		(SIR_ERROR_BASE +  67)
+#define SIR_ERROR_NO_MAIN_METHOD		(SIR_ERROR_BASE +  68)
+#define SIR_ERROR_MEMBER_MAIN_NOT_A_METHOD	(SIR_ERROR_BASE +  69)
+#define SIR_ERROR_MAIN_METHOD_TYPE_MISMATCH	(SIR_ERROR_BASE +  70)
+#define SIR_ERROR_NO_PORTS_ALLOWED_FOR_MAIN	(SIR_ERROR_BASE +  71)
+#define SIR_ERROR_NO_INTERFACES_ALLOWED_FR_MAIN	(SIR_ERROR_BASE +  72)
+#define SIR_ERROR_COMPOUND_STMNT_NOT_CLEAN	(SIR_ERROR_BASE +  73)
 #define SIR_ERROR_ILLEGAL_INTERFACE_MEMBER	(SIR_ERROR_BASE +  74)
 #define SIR_ERROR_CALL_OF_NON_FUNCTION		(SIR_ERROR_BASE +  75)
 #define SIR_ERROR_INVALID_OPERAND_FOR_CONCAT	(SIR_ERROR_BASE +  76)
@@ -172,10 +174,48 @@
 #define SIR_ERROR_CONSTRAINT_LABEL_NOT_IN_DO	(SIR_ERROR_BASE + 175)
 #define SIR_ERROR_CONSTRAINT_ALREADY_EXISTS	(SIR_ERROR_BASE + 176)
 #define SIR_ERROR_CREATED_INTERFACE_NOT_FOUND	(SIR_ERROR_BASE + 177)
-#define SIR_ERROR_MODIFIABLE_LVALUE		(SIR_ERROR_BASE + 196)
-#define SIR_ERROR_PORT_WRITE_ACCESS		(SIR_ERROR_BASE + 197)
-#define SIR_ERROR_PORT_READ_ACCESS		(SIR_ERROR_BASE + 198)
-
+#define SIR_ERROR_MODIFIABLE_LVALUE		(SIR_ERROR_BASE + 181)
+#define SIR_ERROR_PORT_WRITE_ACCESS		(SIR_ERROR_BASE + 182)
+#define SIR_ERROR_PORT_READ_ACCESS		(SIR_ERROR_BASE + 183)
+#define SIR_ERROR_REDEFINITION_OF_STATE         (SIR_ERROR_BASE + 187)
+#define SIR_ERROR_TYPE_BUFFERED_SUBTYPE         (SIR_ERROR_BASE + 188)
+#define SIR_ERROR_TYPE_FUNCTION_SUBTYPE         (SIR_ERROR_BASE + 189)
+#define SIR_ERROR_BUFFERED_LOCAL_USERTYPE       (SIR_ERROR_BASE + 190)
+#define SIR_ERROR_ILLEGAL_BUFFER_CLOCK_SPEC     (SIR_ERROR_BASE + 191)
+#define SIR_ERROR_MISSING_BUFFER_CLOCK_SPEC     (SIR_ERROR_BASE + 192)
+#define SIR_ERROR_LVALUE_REQUIRED_FOR_ADDR_OP   (SIR_ERROR_BASE + 193)
+#define SIR_ERROR_INVALID_ARGUMENT_FOR_ADDR_OP  (SIR_ERROR_BASE + 194)
+#define SIR_ERROR_INVALID_OPERAND_FOR_COMP_OP   (SIR_ERROR_BASE + 195)
+#define SIR_ERROR_INVALID_TYPE_FOR_ASSIGN_OP    (SIR_ERROR_BASE + 196)
+#define SIR_ERROR_INVALID_TYPE_CAST             (SIR_ERROR_BASE + 197)
+#define SIR_ERROR_ILLEGAL_SIZEOF_SIGNAL         (SIR_ERROR_BASE + 198)
+#define SIR_ERROR_ILLEGAL_SIZEOF_BUFFER         (SIR_ERROR_BASE + 199)
+#define SIR_ERROR_PIPED_SIGNAL_OR_BUFFER        (SIR_ERROR_BASE + 200)
+#define SIR_ERROR_PIPED_ARRAY_OF_UNKNOWN_SIZE   (SIR_ERROR_BASE + 201)
+#define SIR_ERROR_ARRAY_SIZE_REQUIRED           (SIR_ERROR_BASE + 202)
+#define SIR_ERROR_EVENT_EXPRESSION              (SIR_ERROR_BASE + 207)
+#define SIR_ERROR_PORT_MAPPING_MISMATCH_11      (SIR_ERROR_BASE + 208)
+#define SIR_ERROR_QUALIFIED_BUFFERED_TYPE       (SIR_ERROR_BASE + 209)
+#define SIR_ERROR_QUALIFIED_PIPED_TYPE          (SIR_ERROR_BASE + 210)
+#define SIR_ERROR_GOTO_LABEL_IN_FSMD            (SIR_ERROR_BASE + 211)
+#define SIR_ERROR_REDEFINITION_OF_FSMD_STATE    (SIR_ERROR_BASE + 212)
+#define SIR_ERROR_UNDEFINED_FSMD_STATE          (SIR_ERROR_BASE + 213)
+#define SIR_ERROR_SYMBOL_NOT_VISIBLE_IN_SCOPE   (SIR_ERROR_BASE + 216)
+#define SIR_ERROR_USERTYPE_NOT_VISIBLE_IN_SCOPE (SIR_ERROR_BASE + 217)
+#define SIR_ERROR_INVALID_DELAYED_ASSIGNMENT_1  (SIR_ERROR_BASE + 218)
+#define SIR_ERROR_INVALID_DELAYED_ASSIGNMENT_2  (SIR_ERROR_BASE + 219)
+#define SIR_ERROR_BREAK_OUTSIDE_LOOP_OR_SWITCH  (SIR_ERROR_BASE + 220)
+#define SIR_ERROR_CONTINUE_OUTSIDE_LOOP         (SIR_ERROR_BASE + 221)
+#define SIR_ERROR_CASE_OUTSIDE_SWITCH           (SIR_ERROR_BASE + 222)
+#define SIR_ERROR_DEFAULT_OUTSIDE_SWITCH        (SIR_ERROR_BASE + 223)
+#define SIR_ERROR_CASE_NOT_2_BELOW_SWITCH       (SIR_ERROR_BASE + 224)
+#define SIR_ERROR_DFLT_NOT_2_BELOW_SWITCH       (SIR_ERROR_BASE + 225)
+#define SIR_ERROR_BREAK_NOT_4_BELOW_SWITCH      (SIR_ERROR_BASE + 226)
+#define SIR_ERROR_BREAK_NOT_LAST_IN_CASE        (SIR_ERROR_BASE + 227)
+#define SIR_ERROR_SWITCH_BODY_NOT_COMPOUND      (SIR_ERROR_BASE + 228)
+#define SIR_ERROR_NOT_CASE_DEFAULT_IN_SWITCH    (SIR_ERROR_BASE + 229)
+#define SIR_ERROR_CASE_NOT_ENDING_WITH_BREAK    (SIR_ERROR_BASE + 230)
+#define SIR_ERROR_MULTIPLE_DEFAULT_IN_SWITCH    (SIR_ERROR_BASE + 231)
 
 /*** exported variables *************************************************/
 
