@@ -4,10 +4,19 @@
 /* Author: Rainer Doemer			first version: 06/24/98 */
 /************************************************************************/
 
-/* last update: 05/31/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 03/14/03 RD	added C++ code generation for 'signal' and 'buffered' types
+ * 10/18/02 RD	handle basics of port maps with SIR_TYPE_SIGNAL, SIR_TYPE_BUFFER
+ * 08/26/02 RD	fixed semantics of XxxDirectMapping
+ * 08/16/02 RD	allow behaviors to be mapped onto suitable interface-type ports
+ * 05/08/02 RD	bug fix in SIR_PortMap::New(DirectMapping)
+ * 11/21/01 RD	took out default arguments from function definitions
+ * 11/13/01 RD	added line wrapping in code generation
+ * 11/08/01 RD	switched code generation to use GL_IO layer
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/27/01 RD	additional mapping checking against directions of class ports
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
@@ -89,6 +98,24 @@ sir_portmap *SIR_PortMap::New(	/* create a new direct port mapping (#2b) */
 
 /* DirectMapping may be NULL meaning open connection */
 
+if (  (DirectMapping)
+    &&(DirectMapping->Type->TypeClass() == SIR_TYPECLASS_BITVECTOR))
+   { /* bug fix 05/08/02, RD: bitvectors need bounds!  */
+     /* added   10/17/02, RD: signals and buffers too! */
+     sir_type	*BitType;
+
+     BitType = DirectMapping->Type;
+     if (  (BitType->Type == SIR_TYPE_SIGNAL)
+	 ||(BitType->Type == SIR_TYPE_BUFFER))
+	{ BitType = BitType->SubType;
+	 } /* fi */
+     return(new SIR_PortMap(
+		new SIR_BitSlices(
+			new SIR_BitSlice(DirectMapping,
+					BitType->LeftBound,
+					BitType->RightBound))));
+    } /* fi */
+
 return(new SIR_PortMap(
 		new SIR_BitSlices(
 			new SIR_BitSlice(DirectMapping))));
@@ -125,7 +152,7 @@ if (BitSlices)
 
 
 void SIR_PortMap::WriteCC_Decl(		/* generates C++ source code (part 1) */
-	FILE		*File,
+	GL_HANDLE	*IO,
 	sir_type	*PortType,
 	int		*BitBusNum,
 	int		*OpenPortNum,
@@ -136,13 +163,14 @@ sir_bitslice	*BitSlice;
 sir_types	*TypeTable;
 sir_type	*ConstType;
 
-assert(File != NULL);
+assert(IO != NULL);
 assert(PortType != NULL);
 
 if (IsOpenMapping())		/* create dummy for open mappings */
-   { SIR_LineInfo::WriteNL(File);
+   { SIR_LineInfo::WriteNL(IO);
      Buffer.form(SIR_CXX_OPEN_PORT_NAME, (*OpenPortNum)++);
-     fprintf(File, "%s;", PortType->PrettyString(&Buffer, FALSE, TRUE, FALSE));
+     IO->PutS(PortType->PrettyString(&Buffer, FALSE, TRUE, FALSE));
+     IO->PutC(';');
      return;
     } /* fi */
 
@@ -151,7 +179,7 @@ BitSlice = BitSlices->First();	/* create dummies for constant mappings */
 while(BitSlice)
    { if (BitSlice->Constant)
 	{ assert(BitSlice->Symbol == NULL);
-	  SIR_LineInfo::WriteNL(File);
+	  SIR_LineInfo::WriteNL(IO);
 	  Buffer.form(SIR_CXX_CONST_PORT_NAME, (*ConstPortNum)++);
 	  TypeTable = PortType->GetTable();
 	  assert(TypeTable != NULL);
@@ -160,8 +188,13 @@ while(BitSlice)
 	  ConstType = TypeTable->FindOrInsertModifiedType(ConstType,
 							false); /* no const */
 	  assert(ConstType != NULL);
-	  fprintf(File, "%s;", ConstType->PrettyString(&Buffer,
-							FALSE, TRUE, FALSE));
+	  if (  (PortType->Type == SIR_TYPE_BUFFER)
+	      ||(PortType->Type == SIR_TYPE_SIGNAL))
+	     { ConstType = TypeTable->FindOrInsert(PortType->Type, ConstType);
+	       assert(ConstType != NULL);
+	      } /* fi */
+	  IO->PutS(ConstType->PrettyString(&Buffer, FALSE, TRUE, FALSE));
+	  IO->PutC(';');
 	 } /* fi */
      BitSlice = BitSlice->Succ();
     } /* elihw */
@@ -171,13 +204,33 @@ if (IsDirectlyMappable(PortType))	/* determine mapping type */
     } /* fi */
 else
    { /* indirect mapping required */
-     SIR_LineInfo::WriteNL(File);
-     fprintf(File, SIR_BIT_BUS_CLASS_NAME "<"
+     SIR_LineInfo::WriteNL(IO);
+     if (PortType->Type == SIR_TYPE_BUFFER)
+	{ IO->PrintF(SIR_SIM_BUF_BUS_NAME "<"
+			SIR_BIT_LENGTH_NAME "(%d,%d),%s> ",
+			PortType->SubType->LeftBound,
+			PortType->SubType->RightBound,
+			((PortType->SubType->Type == SIR_TYPE_BIT) ?
+				SIR_BIT_SIGNED_FLAG : SIR_BIT_UNSIGNED_FLAG));
+	 } /* fi */
+     else
+	{ if (PortType->Type == SIR_TYPE_SIGNAL)
+	     { IO->PrintF(SIR_SIM_SIG_BUS_NAME "<"
+			SIR_BIT_LENGTH_NAME "(%d,%d),%s> ",
+			PortType->SubType->LeftBound,
+			PortType->SubType->RightBound,
+			((PortType->SubType->Type == SIR_TYPE_BIT) ?
+				SIR_BIT_SIGNED_FLAG : SIR_BIT_UNSIGNED_FLAG));
+	      } /* fi */
+	  else
+	     { IO->PrintF(SIR_BIT_BUS_CLASS_NAME "<"
 			SIR_BIT_LENGTH_NAME "(%d,%d),%s> ",
 			PortType->LeftBound, PortType->RightBound,
 			((PortType->Type == SIR_TYPE_BIT) ?
 				SIR_BIT_SIGNED_FLAG : SIR_BIT_UNSIGNED_FLAG));
-     fprintf(File, SIR_CXX_SLICED_BUS_NAME ";", (*BitBusNum)++);
+	      } /* esle */
+	 } /* esle */
+     IO->PrintF(SIR_CXX_SLICED_BUS_NAME ";", (*BitBusNum)++);
     } /* esle */
 
 } /* end of SIR_PortMap::WriteCC_Decl */
@@ -201,7 +254,23 @@ int		StoreConstPortNum;
 assert(PortType != NULL);
 
 if (IsOpenMapping())		/* initialize dummy for open mappings */
-   { Buffer.form(SIR_CXX_OPEN_PORT_NAME "(0)", (*OpenPortNum)++);
+   { if (  (PortType->Type == SIR_TYPE_EVENT)
+	 ||(PortType->Type == SIR_TYPE_SIGNAL))
+	{ /* event/signal dummies don't need initialization */
+	  (*OpenPortNum)++;
+	  return(NULL);
+	 } /* fi */
+     else
+	{ if (PortType->Type == SIR_TYPE_BUFFER)
+	     { Buffer.form(SIR_CXX_OPEN_PORT_NAME "("
+						SIR_SIM_NO_SIG_NAME ", 0)",
+			(*OpenPortNum)++);
+	      } /* fi */
+	  else
+	     { /* although they are never read, nicely initialize the dummies */
+	       Buffer.form(SIR_CXX_OPEN_PORT_NAME "(0)", (*OpenPortNum)++);
+	      } /* esle */
+	 } /* esle */
      return(Buffer.chars());
     } /* fi */
 
@@ -216,8 +285,15 @@ while(BitSlice)
 	  if (Buffer.length() > 0)
 	     { Buffer += ", ";
 	      } /* fi */
-	  TmpString.form(SIR_CXX_CONST_PORT_NAME "(%s)",
+	  if (PortType->Type == SIR_TYPE_BUFFER)
+	     { TmpString.form(SIR_CXX_CONST_PORT_NAME "("
+						SIR_SIM_NO_SIG_NAME ", %s)",
 			(*ConstPortNum)++, BitSlice->Constant->Print(TRUE));
+	      } /* fi */
+	  else
+	     { TmpString.form(SIR_CXX_CONST_PORT_NAME "(%s)",
+			(*ConstPortNum)++, BitSlice->Constant->Print(TRUE));
+	      } /* esle */
 	  Buffer += TmpString;
 	 } /* fi */
      BitSlice = BitSlice->Succ();
@@ -259,13 +335,30 @@ if (! IsDirectlyMappable(PortType))	/* indirect mapping required */
 	     { Buffer += "->" SIR_BIT_BUS_CONCAT_NAME "(";
 	      } /* fi */
 	  if (BitSlice->Symbol)
-	     { TmpString.form("%s%s." SIR_BIT_BUS_SLICE_NAME "(%d,%d, %d,%d)",
+	     { MapType = BitSlice->Symbol->Type;
+	       if (  (MapType->Type == SIR_TYPE_SIGNAL)
+		   ||(MapType->Type == SIR_TYPE_BUFFER))
+		  { TmpString.form("%s%s.%s(%d,%d, %d,%d)",
 				BitSlice->Symbol->Name.chars(),
 				PipedAccess.chars(),
-				BitSlice->Symbol->Type->LeftBound,
-				BitSlice->Symbol->Type->RightBound,
+				MapType->Type == SIR_TYPE_SIGNAL ?
+					SIR_SIM_SIGBUS_SLC_NAME :
+					SIR_SIM_BUFBUS_SLC_NAME,
+				MapType->SubType->LeftBound,
+				MapType->SubType->RightBound,
 				BitSlice->LeftBound,
 				BitSlice->RightBound);
+		   } /* fi */
+	       else
+		  { TmpString.form("%s%s." SIR_BIT_BUS_SLICE_NAME
+							"(%d,%d, %d,%d)",
+				BitSlice->Symbol->Name.chars(),
+				PipedAccess.chars(),
+				MapType->LeftBound,
+				MapType->RightBound,
+				BitSlice->LeftBound,
+				BitSlice->RightBound);
+		   } /* esle */
 	       Buffer += TmpString;
 	      } /* fi */
 	  else
@@ -273,9 +366,13 @@ if (! IsDirectlyMappable(PortType))	/* indirect mapping required */
 	       MapType = PortType->GetTable()->FindOrInsert(BitSlice->Constant);
 	       MapLeft = MapType->LeftBound;
 	       MapRight = MapType->RightBound;
-	       TmpString.form(SIR_CXX_CONST_PORT_NAME "."
-				SIR_BIT_BUS_SLICE_NAME "(%d,%d, %d,%d)",
+	       TmpString.form(SIR_CXX_CONST_PORT_NAME ".%s(%d,%d, %d,%d)",
 				(*ConstPortNum)++,
+				PortType->Type == SIR_TYPE_SIGNAL ?
+					SIR_SIM_SIGBUS_SLC_NAME :
+					(PortType->Type == SIR_TYPE_BUFFER ?
+						SIR_SIM_BUFBUS_SLC_NAME :
+						SIR_BIT_BUS_SLICE_NAME),
 				MapLeft, MapRight,
 				MapLeft, MapRight);
 	       Buffer += TmpString;
@@ -299,7 +396,7 @@ else
 
 
 void SIR_PortMap::WriteCC(		/* generates C++ source code (part 3) */
-	FILE		*File,
+	GL_HANDLE	*IO,
 	sir_type	*PortType,
 	int		*BitBusNum,
 	int		*OpenPortNum,
@@ -307,11 +404,11 @@ void SIR_PortMap::WriteCC(		/* generates C++ source code (part 3) */
 {
 sir_bitslice	*BitSlice;
 
-assert(File != NULL);
+assert(IO != NULL);
 assert(PortType != NULL);
 
 if (IsOpenMapping())		/* map to dummy for open mappings */
-   { fprintf(File, SIR_CXX_OPEN_PORT_NAME, (*OpenPortNum)++);
+   { IO->PrintF(SIR_CXX_OPEN_PORT_NAME, (*OpenPortNum)++);
      return;
     } /* fi */
 
@@ -320,7 +417,7 @@ if (IsDirectlyMappable(PortType))	/* determine mapping type */
      BitSlice = BitSlices->First();
      if (BitSlice->Constant)
 	{ assert(BitSlice->Symbol == NULL);
-	  fprintf(File, SIR_CXX_CONST_PORT_NAME, (*ConstPortNum)++);
+	  IO->PrintF(SIR_CXX_CONST_PORT_NAME, (*ConstPortNum)++);
 	  return;
 	 } /* fi */
      assert(BitSlice->Symbol != NULL);
@@ -331,14 +428,14 @@ if (IsDirectlyMappable(PortType))	/* determine mapping type */
 		  { assert(FALSE);	/* type checked! */
 		   }
 	       case SIR_PORT_IN:
-		  { fputs(BitSlice->Symbol->Name.chars(), File);
-		    fprintf(File, SIR_SIM_PIPED_IN_ACCESS(
+		  { IO->PutS(BitSlice->Symbol->Name);
+		    IO->PrintF(SIR_SIM_PIPED_IN_ACCESS(
 					BitSlice->Symbol->PipeStages));
 		    break;
 		   }
 	       case SIR_PORT_OUT:
-		  { fputs(BitSlice->Symbol->Name.chars(), File);
-		    fprintf(File, SIR_SIM_PIPED_OUT_ACCESS(
+		  { IO->PutS(BitSlice->Symbol->Name);
+		    IO->PrintF(SIR_SIM_PIPED_OUT_ACCESS(
 					BitSlice->Symbol->PipeStages));
 		    break;
 		   }
@@ -348,12 +445,12 @@ if (IsDirectlyMappable(PortType))	/* determine mapping type */
 	      } /* hctiws */
 	 } /* fi */
      else
-	{ fputs(BitSlice->Symbol->Name.chars(), File);
+	{ IO->PutS(BitSlice->Symbol->Name);
 	 } /* esle */
     } /* fi */
 else
    { /* indirect mapping required */
-     fprintf(File, SIR_CXX_SLICED_BUS_NAME, (*BitBusNum)++);
+     IO->PrintF(SIR_CXX_SLICED_BUS_NAME, (*BitBusNum)++);
     } /* esle */
 
 } /* end of SIR_PortMap::WriteCC */
@@ -394,6 +491,11 @@ int		PortLeft,
 		MapLeft,
 		MapRight;
 
+// note: this code assumes that type checking already has taken place
+//       for this mapping and was successful; here, we only need to
+//       find out which method of implementation for the mapping to
+//       choose
+
 assert(PortType != NULL);
 assert(BitSlices != NULL);
 
@@ -407,29 +509,42 @@ if (  (! BitSlice->Symbol)
 if (BitSlice->Symbol)
    { assert(BitSlice->Constant == NULL);
      MapType = BitSlice->Symbol->Type;
+     assert(MapType != NULL);
      MapLeft = BitSlice->LeftBound;
      MapRight = BitSlice->RightBound;
     } /* fi */
 else
    { assert(BitSlice->Constant != NULL);
      MapType = PortType->GetTable()->FindOrInsert(BitSlice->Constant);
+     assert(MapType != NULL);
      MapLeft = MapType->LeftBound;
      MapRight = MapType->RightBound;
     } /* esle */
-assert(MapType != NULL);
 
 if (PortType->TypeClass() != SIR_TYPECLASS_BITVECTOR)
-   { assert(BitSlices->NumElements() == 1);			/* type check */
-     assert(MapType->TypeClass() != SIR_TYPECLASS_BITVECTOR);	/* type check */
+   { assert(BitSlices->NumElements() == 1);
+     assert(MapType->TypeClass() != SIR_TYPECLASS_BITVECTOR);
      return(TRUE);
     } /* fi */
 
 	/* bitvector mapping */
 
+assert(MapType->TypeClass() == SIR_TYPECLASS_BITVECTOR);
+assert(PortType->TypeClass() == SIR_TYPECLASS_BITVECTOR);
+
 if (BitSlices->NumElements() != 1)
    { return(FALSE);	/* concatenated bitslices */
     } /* fi */
 
+if (  (MapType->Type == SIR_TYPE_SIGNAL)
+    ||(MapType->Type == SIR_TYPE_BUFFER))
+   { MapType = MapType->SubType;
+    } /* fi */
+if (  (PortType->Type == SIR_TYPE_SIGNAL)
+    ||(PortType->Type == SIR_TYPE_BUFFER))
+   { PortType = PortType->SubType;
+    } /* fi */
+
 if (  (MapLeft != MapType->LeftBound)
     ||(MapRight != MapType->RightBound))
    { return(FALSE);	/* sliced and/or reversed mapping */
@@ -452,11 +567,13 @@ return(SIR_BIT_DIRECTLY_MAPPABLE(	/* mapping supported by bit library */
 ERROR SIR_PortMap::Check(	/* semantically check the port mapping */
 	sir_type_ptr	*Port,
 	unsigned int	PortNum,
-	const char	*SourceFile = NULL,	/* default: no warnings */
-	unsigned int	SourceLine = 0)
+	const char	*SourceFile /* = NULL */, /* default: no warnings */
+	unsigned int	SourceLine /* = 0 */)
 {
 sir_bitslice	*BitSlice;
-sir_type	*PortMapType;
+sir_type	*PortMapType,
+		*PortBitType,
+		*PortMapBitType;
 int		BusWidth;
 
 assert(Port != NULL);
@@ -468,17 +585,46 @@ if (Port->Type->TypeClass() == SIR_TYPECLASS_BITVECTOR)
      while(BitSlice)
 	{ if (BitSlice->Symbol)
 	     { assert(BitSlice->Constant == NULL);
-	       if (BitSlice->Symbol->Type->TypeClass() !=
-						SIR_TYPECLASS_BITVECTOR)
+	       PortMapType = BitSlice->Symbol->Type;
+	       if (PortMapType->TypeClass() != SIR_TYPECLASS_BITVECTOR)
 		  { SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
 							GL_ERROR_MSG_NEWLINE
 				"bitvector type expected",
 				PortNum);
 		    return(SIR_ERROR_PORT_MAPPING_MISMATCH_5);
 		   } /* fi */
+	       if (Port->Type->Type == SIR_TYPE_SIGNAL)
+		  { if (PortMapType->Type != SIR_TYPE_SIGNAL)
+		       { SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
+							GL_ERROR_MSG_NEWLINE
+				"'signal' type expected",
+				PortNum);
+			 return(SIR_ERROR_PORT_MAPPING_MISMATCH_11);
+			} /* fi */
+		   } /* fi */
+	       if (Port->Type->Type == SIR_TYPE_BUFFER)
+		  { if (PortMapType->Type != SIR_TYPE_BUFFER)
+		       { SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
+							GL_ERROR_MSG_NEWLINE
+				"'buffered' type expected",
+				PortNum);
+			 return(SIR_ERROR_PORT_MAPPING_MISMATCH_11);
+			} /* fi */
+		   } /* fi */
+	       if (  (Port->Type->Type != SIR_TYPE_SIGNAL)
+		   &&(Port->Type->Type != SIR_TYPE_BUFFER))
+		  { if (  (PortMapType->Type == SIR_TYPE_SIGNAL)
+			||(PortMapType->Type == SIR_TYPE_BUFFER))
+		       { SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
+							GL_ERROR_MSG_NEWLINE
+				"non-'signal'/non-'buffered' type expected",
+				PortNum);
+			 return(SIR_ERROR_PORT_MAPPING_MISMATCH_11);
+			} /* fi */
+		   } /* fi */
 	       if (  (BitSlice->Symbol->StorageClass == SIR_STORAGE_PIPED)
 		   &&(  (Port->Type->Direction != SIR_PORT_IN)
-		   &&(Port->Type->Direction != SIR_PORT_OUT)))
+		      &&(Port->Type->Direction != SIR_PORT_OUT)))
 		  { SIR_ErrMsg.form("Illegal port mapping for port %u:"
 							GL_ERROR_MSG_NEWLINE
 				"mapping to piped variable requires "
@@ -486,7 +632,6 @@ if (Port->Type->TypeClass() == SIR_TYPECLASS_BITVECTOR)
 				PortNum);
 		    return(SIR_ERROR_PORT_MAPPING_MISMATCH_6);
 		   } /* fi */
-	       PortMapType = BitSlice->Symbol->Type;
 	       if (  (  (Port->Type->Direction == SIR_PORT_IN)
 		      &&(PortMapType->Direction == SIR_PORT_OUT))
 		   ||(  (Port->Type->Direction == SIR_PORT_INOUT)
@@ -541,44 +686,55 @@ if (Port->Type->TypeClass() == SIR_TYPECLASS_BITVECTOR)
 	      } /* esle */
 	  BitSlice = BitSlice->Succ();
 	 } /* elihw */
-     if (ABS(Port->Type->LeftBound - Port->Type->RightBound) + 1 != BusWidth)
+     if (  (Port->Type->Type == SIR_TYPE_SIGNAL)
+	 ||(Port->Type->Type == SIR_TYPE_BUFFER))
+	{ PortBitType = Port->Type->SubType;
+	 } /* fi */
+     else
+	{ PortBitType = Port->Type;
+	 } /* esle */
+     if (ABS(PortBitType->LeftBound - PortBitType->RightBound) + 1 != BusWidth)
 	{ SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
 							GL_ERROR_MSG_NEWLINE
 			"bus width (%d) does not match port width (%d)",
 			PortNum, BusWidth,
-			ABS(Port->Type->LeftBound - Port->Type->RightBound) +1);
+			ABS(PortBitType->LeftBound-PortBitType->RightBound)+1);
 	  return(SIR_ERROR_PORT_MAPPING_MISMATCH_4);
 	 } /* fi */
      if (  (SourceFile)
 	 &&(BitSlices->NumElements() == 1)
-	 &&(BitSlices->First()->Symbol)
-	 &&(BitSlices->First()->Symbol->Type->Type != Port->Type->Type))
-	{ if (Port->Type->Type == SIR_TYPE_BIT)
-	     { assert(BitSlices->First()->Symbol->Type->Type == SIR_TYPE_UBIT);
-	       GL_PrintWarningFmt(GL_WARN_INFORMATIVE,
+	 &&(BitSlices->First()->Symbol))
+	{ PortMapBitType = BitSlices->First()->Symbol->Type;
+	  if (  (PortMapBitType == SIR_TYPE_SIGNAL)
+	      ||(PortMapBitType == SIR_TYPE_BUFFER))
+	     { PortMapBitType = PortMapBitType->SubType;
+	      } /* fi */
+	  if (  (PortBitType->Type == SIR_TYPE_BIT)
+	      &&(PortMapBitType->Type == SIR_TYPE_UBIT))
+	     { GL_PrintWarningFmt(GL_WARN_INFORMATIVE,
 			"In file '%s', line %u," GL_WARN_MSG_NEWLINE
 			"signed bitvector port %u mapped to unsigned type",
 			SourceFile, SourceLine, PortNum);
 	      } /* fi */
-	  else
-	     { assert(BitSlices->First()->Symbol->Type->Type == SIR_TYPE_BIT);
-	       GL_PrintWarningFmt(GL_WARN_INFORMATIVE,
+	  if (  (PortBitType->Type == SIR_TYPE_UBIT)
+	      &&(PortMapBitType->Type == SIR_TYPE_BIT))
+	     { GL_PrintWarningFmt(GL_WARN_INFORMATIVE,
 			"In file '%s', line %u," GL_WARN_MSG_NEWLINE
 			"unsigned bitvector port %u mapped to signed type",
 			SourceFile, SourceLine, PortNum);
-	      } /* esle */
+	      } /* fi */
 	 } /* fi */
      if (  (SourceFile)
 	 &&(BitSlices->NumElements() == 1)
 	 &&(BitSlices->First()->Constant))
-	{ if (  (Port->Type->Type == SIR_TYPE_BIT)
+	{ if (  (PortBitType->Type == SIR_TYPE_BIT)
 	      &&(BitSlices->First()->Constant->Type == SIR_CONST_UBIT))
 	     { GL_PrintWarningFmt(GL_WARN_INFORMATIVE,
 			"In file '%s', line %u," GL_WARN_MSG_NEWLINE
 			"signed bitvector port %u mapped to unsigned constant",
 			SourceFile, SourceLine, PortNum);
 	      } /* fi */
-	  if (  (Port->Type->Type == SIR_TYPE_UBIT)
+	  if (  (PortBitType->Type == SIR_TYPE_UBIT)
 	      &&(BitSlices->First()->Constant->Type == SIR_CONST_BIT))
 	     { GL_PrintWarningFmt(GL_WARN_INFORMATIVE,
 			"In file '%s', line %u," GL_WARN_MSG_NEWLINE
@@ -599,6 +755,35 @@ else
      assert(BitSlice != NULL);
      if (BitSlice->Symbol)
 	{ PortMapType = BitSlice->Symbol->Type;
+	  if (Port->Type->Type == SIR_TYPE_SIGNAL)
+	     { if (PortMapType->Type != SIR_TYPE_SIGNAL)
+		  { SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
+							GL_ERROR_MSG_NEWLINE
+				"'signal' type expected",
+				PortNum);
+		    return(SIR_ERROR_PORT_MAPPING_MISMATCH_11);
+		   } /* fi */
+	      } /* fi */
+	  if (Port->Type->Type == SIR_TYPE_BUFFER)
+	     { if (PortMapType->Type != SIR_TYPE_BUFFER)
+		  { SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
+							GL_ERROR_MSG_NEWLINE
+				"'buffered' type expected",
+				PortNum);
+		    return(SIR_ERROR_PORT_MAPPING_MISMATCH_11);
+		   } /* fi */
+	      } /* fi */
+	  if (  (Port->Type->Type != SIR_TYPE_SIGNAL)
+	      &&(Port->Type->Type != SIR_TYPE_BUFFER))
+	     { if (  (PortMapType->Type == SIR_TYPE_SIGNAL)
+		   ||(PortMapType->Type == SIR_TYPE_BUFFER))
+		  { SIR_ErrMsg.form("Mismatch in port mapping for port %u:"
+							GL_ERROR_MSG_NEWLINE
+				"non-'signal'/non-'buffered' type expected",
+				PortNum);
+		    return(SIR_ERROR_PORT_MAPPING_MISMATCH_11);
+		   } /* fi */
+	      } /* fi */
 	  if (  (  (Port->Type->Direction == SIR_PORT_IN)
 		 &&(PortMapType->Direction == SIR_PORT_OUT))
 	      ||(  (Port->Type->Direction == SIR_PORT_INOUT)
@@ -630,6 +815,11 @@ else
 		  { PortMapType =	/* mapped onto constants    */
 			PortMapType->Modified(false);	/* no const flag! */
 		   } /* fi */
+	       if (  (Port->Type->Type == SIR_TYPE_SIGNAL)	// allow const.
+		   ||(Port->Type->Type == SIR_TYPE_BUFFER))	// for sig/buf.
+		  { PortMapType = PortMapType->GetTable()->FindOrInsert(
+						Port->Type->Type, PortMapType);
+		   } /* fi */
 	      } /* fi */
 	  else
 	     { /* open connection */
@@ -655,8 +845,8 @@ else
 				PortMapType, Port->Type,
 				true))	/* (ignore direction) */
 	{ if (  (Port->Type->Type != SIR_TYPE_INTERFACE)
-	      ||(PortMapType->Type != SIR_TYPE_CHANNEL)
-	/* note: channels can be mapped to interface ports, behaviors cannot! */
+	      ||(  (PortMapType->Type != SIR_TYPE_CHANNEL)
+	         &&(PortMapType->Type != SIR_TYPE_BEHAVIOR)) /* 08/16/02, RD */
 	      ||(! PortMapType->ClassSymbol->Interfaces->Find(
 						Port->Type->ClassSymbol)))
 	     { SIR_ErrMsg.form("Type mismatch in port mapping for port %u",
@@ -691,7 +881,7 @@ return(SIR_ERROR_NO_ERROR);
 
 
 SIR_PortMaps::SIR_PortMaps(		/* constructor #1 */
-	sir_portmap	*FirstEntry = NULL) :
+	sir_portmap	*FirstEntry /* = NULL */) :
 		SIR_List<SIR_PortMap>(FirstEntry)
 {
 
@@ -760,8 +950,8 @@ while(PortMap)
 
 ERROR SIR_PortMaps::Check(	/* semantically check the port mappings */
 	sir_type_ptrs	*PortList,
-	const char	*SourceFile = NULL,	/* default: no warnings */
-	unsigned int	SourceLine = 0)
+	const char	*SourceFile /* = NULL */, /* default: no warnings */
+	unsigned int	SourceLine /* = 0 */)
 {
 sir_portmap	*PortMap;
 sir_type_ptr	*Port;
@@ -799,7 +989,7 @@ return(SIR_ERROR_NO_ERROR);
 
 
 ERROR SIR_PortMaps::WriteSC(		/* (re-) generates SpecC source code */
-	FILE		*File)
+	GL_HANDLE	*IO)
 {
 sir_portmap	*PortMap;
 sir_bitslice	*BitSlice;
@@ -807,47 +997,49 @@ sir_bitslice	*BitSlice;
 assert(this != NULL);	/* always exists */
 
 if (! Empty())
-   { fputc('(', File);
+   { IO->PutC('(');
+     IO->TabStepUp();
      PortMap = First();
      while(PortMap)
 	{ assert(PortMap->BitSlices != NULL);
 	  BitSlice = PortMap->BitSlices->First();
 	  while(BitSlice)
 	     { if (BitSlice->Symbol)
-		  { fputs(BitSlice->Symbol->Name.chars(), File);
+		  { IO->PutS(BitSlice->Symbol->Name);
 		    if (BitSlice->Symbol->Type->TypeClass()
 					== SIR_TYPECLASS_BITVECTOR)
-		       { fprintf(File, "[%d:%d]", BitSlice->LeftBound,
+		       { IO->PrintF("[%d:%d]", BitSlice->LeftBound,
 						BitSlice->RightBound);
 			} /* fi */
 		   } /* fi */
 	       else
 		  { if (BitSlice->Constant)
-		       { fputs("(", File);
-			 fputs(BitSlice->Constant->Print(), File);
-			 fputs(")", File);
+		       { IO->PutC('(');
+			 IO->PutS(BitSlice->Constant->Print());
+			 IO->PutC(')');
 			} /* fi */
 		    else
-		       { fputs("/* open */", File);
+		       { IO->PutS("/* open */");
 			} /* fi */
 		   } /* esle */
 	       BitSlice = BitSlice->Succ();
 	       if (BitSlice)
-		  { fputs(" @ ", File);
+		  { IO->PutS(" @ ");
+		    SIR_LineInfo::WrapLine(IO);
 		   } /* fi */
 	      } /* elihw */
 	  PortMap = PortMap->Succ();
 	  if (PortMap)
-	     { fputs(", ", File);
+	     { IO->PutS(", ");
+	       SIR_LineInfo::WrapLine(IO);
 	      } /* fi */
 	 } /* elihw */
-     fputc(')', File);
+     IO->TabStepDown();
+     IO->PutC(')');
     } /* fi */
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
 return(SIR_ERROR_NO_ERROR);
@@ -865,8 +1057,8 @@ return(SIR_ERROR_NO_ERROR);
 
 SIR_BitSlice::SIR_BitSlice(		/* constructor #1 (standard) */
 	sir_symbol	*Symbol,	/* NULL for open connection */
-	int		LeftBound = 0,
-	int		RightBound = 0)
+	int		LeftBound /* = 0 */,
+	int		RightBound /* = 0 */)
 {
 
 SIR_BitSlice::Symbol		= Symbol;
@@ -914,9 +1106,10 @@ delete Constant;
 
 sir_bitslice *SIR_BitSlice::New(	/* create a new bitslice #1b */
 	sir_symbol	*Mapping,	/* (returns NULL if SIR_Error) */
-	int		LeftBound = 0,	/* bounds for bitvectors */
-	int		RightBound = 0)
+	int		LeftBound /* = 0 */,	/* bounds for bitvectors */
+	int		RightBound /* = 0 */)
 {
+sir_type	*MapType;
 int		Left,
 		Right;
 
@@ -924,8 +1117,13 @@ int		Left,
 
 if (  (Mapping)
     &&(Mapping->Type->TypeClass() == SIR_TYPECLASS_BITVECTOR))
-   { Left = Mapping->Type->LeftBound;
-     Right = Mapping->Type->RightBound;
+   { MapType = Mapping->Type;
+     if (  (MapType->Type == SIR_TYPE_SIGNAL)
+	 ||(MapType->Type == SIR_TYPE_BUFFER))
+	{ MapType = MapType->SubType;
+	 } /* fi */
+     Left = MapType->LeftBound;
+     Right = MapType->RightBound;
      if (  (  (Left > Right)
 	    &&(  (LeftBound > Left)  || (LeftBound < Right)
 	       ||(RightBound > Left) || (RightBound < Right)))
@@ -972,7 +1170,7 @@ return(SIR_ERROR_NO_ERROR);
 
 
 SIR_BitSlices::SIR_BitSlices(		/* constructor #1 */
-	sir_bitslice	*FirstEntry = NULL) :
+	sir_bitslice	*FirstEntry /* = NULL */) :
 		SIR_List<SIR_BitSlice>(FirstEntry)
 {
 
