@@ -4,10 +4,29 @@
 /* Author: Rainer Doemer			first version: 10/08/97 */
 /************************************************************************/
 
-/* last update:	06/05/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 02/13/03 RD	added support for notes at fsmd states
+ * 02/12/03 RD	bug fix: FindLocallyViaTypeDef() only finds unnamed usertypes
+ * 09/19/02 RD	support for adding source code info during code generation
+ * 06/20/02 RD	bug fix: merge annotations at usertypes during Integrate()
+ * 04/08/02 RD	bug fix: 'import' of anonymous enum definitions
+ * 04/17/02 RD	bug fix: added SIR_UserTypes::NameAnonymousTypes() and
+ *		SIR_UserTypes::CreateNewName() (some anonymous user-types
+ *		were defined multiple times when generating SpecC/C++ code)
+ * 02/20/02 RD	allow 'import' to sort out recursive structures and enums
+ * 02/15/02 RD	allow double import of user-type defs. if they are the same
+ *		(this resolves the long-lasting import-vs-include problem)
+ * 02/15/02 RD	added SIR_UserType::IsSameStructure()
+ * 02/15/02 RD	bug fix: user-type declarations that are later used 'const'
+ *		made 'import' fail with error #2037 (see HeaderFile.sc)
+ * 11/21/01 RD	took out default arguments from function definitions
+ * 11/08/01 RD	switched code generation to use GL_IO layer
+ * 10/11/01 RD	improved overall formatting of generated code
+ * 10/04/01 RD	improved indentation of generated code
  * 06/05/00 RD	applied another bug fix in SIR_UserType::ContainsNested()
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
@@ -50,11 +69,11 @@
 SIR_UserType::SIR_UserType(		/* constructor #1 */
 	SIR_USERTYPE	Class,
 	const char	*Name,
-	sir_type	*Type = NULL,
-	sir_members	*Members = NULL,
-	sir_symbols	*Scope = NULL,
-	sir_notes	*Notes = NULL,
-	sir_symbol	*TypeDef = NULL)
+	sir_type	*Type /* = NULL */,
+	sir_members	*Members /* = NULL */,
+	sir_symbols	*Scope /* = NULL */,
+	sir_notes	*Notes /* = NULL */,
+	sir_symbol	*TypeDef /* = NULL */)
 {
 
 assert(  (Class == SIR_USERTYPE_STRUCT)
@@ -138,25 +157,82 @@ return(Members != NULL);
 } /* end of SIR_UserType::IsDefinition */
 
 
-sir_usertypes *SIR_UserType::GetTable(void)	/* determines its table */
-{						/* (NULL if not in a table) */
+BOOL SIR_UserType::IsSameStructure(	/* checks if structure is the same */
+	sir_usertype	*Other)
+{
+sir_member	*Member1,
+		*Member2;
 
-return((sir_usertypes*) Head());
+assert(Other != NULL);
 
-} /* end of SIR_UserType::GetTable */
-
-
-sir_symbols *SIR_UserType::GetScope(void)	/* determines the scope */
-{						/* (NULL if no links) */
-sir_usertypes	*Table;
-
-if ((Table = GetTable()))
-   { return(Table->Symbols);
+if (this->Color == SIR_RED)	/* loop in recursion? */
+   { return(TRUE);	/* we are checking this already */
     } /* fi */
 
-return(NULL);
+if (this == Other)	/* identity? */
+   { return(TRUE);
+    } /* fi */
 
-} /* end of SIR_UserType::GetScope */
+if (  ((Name) && (!Other->Name))
+    ||((!Name) && (Other->Name)))
+   { return(FALSE);
+    } /* fi */
+if (  (Name)
+    &&(0 != strcmp(Name->chars(), Other->Name->chars())))
+   { return(FALSE);
+    } /* fi */
+if (  ((Members) && (!Other->Members))
+    ||((!Members) && (Other->Members)))
+   { return(FALSE);
+    } /* fi */
+if (! Members)
+   { assert(!Other->Members);
+     return(TRUE);
+    } /* fi */
+
+Member1 = Members->First();
+Member2 = Other->Members->First();
+while(Member1)
+   { if (!Member2)
+	{ return(FALSE);
+	 } /* fi */
+
+     assert(this->Color == SIR_WHITE);
+     this->Color = SIR_RED;	/* mark node to avoid infinite recursion! */
+     if (0 != SIR_Types::TypeCmp(Member1->Type, Member2->Type,	/* possible  */
+						false, true))	/* recursion */
+	{ assert(this->Color == SIR_RED);
+	  this->Color = SIR_WHITE;	/* remove the mark */
+	  return(FALSE);
+	 } /* fi */
+     assert(this->Color == SIR_RED);
+     this->Color = SIR_WHITE;	/* remove the mark */
+
+     if (Member1->Symbol)
+	{ if (  (!Member2->Symbol)
+	      ||(0 != strcmp(Member1->Symbol->Name.chars(),
+				Member2->Symbol->Name.chars())))
+	     { return(FALSE);
+	      } /* fi */
+	 } /* fi */
+     else
+	{ if (Member2->Symbol)
+	     { return(FALSE);
+	      } /* fi */
+	 } /* esle */
+     if (Member1->BitFieldSize != Member2->BitFieldSize)
+	{ return(FALSE);
+	 } /* fi */
+     Member1 = Member1->Succ();
+     Member2 = Member2->Succ();
+    } /* elihw */
+if (Member2)
+   { return(FALSE);
+    } /* fi */
+
+return(TRUE);
+
+} /* end of SIR_UserType::IsSameStructure */
 
 
 BOOL SIR_UserType::ContainsNested(	/* checks for nested struct/union */
@@ -198,6 +274,59 @@ return(FALSE);	/* not found */
 } /* end of SIR_UserType::ContainsNested */
 
 
+BOOL SIR_UserType::LocalUTypeNeedsDef(	/* check whether a user-type member */
+	sir_member	*Member)	/* needs a local definition	    */
+{
+sir_member	*PrevMember;
+
+assert(Member != NULL);
+assert(Class != SIR_USERTYPE_ENUM);	/* only applies to struct/union */
+assert(Scope != NULL);			/* union/struct have local scope */
+
+if (Member->Type->UserType)	/* is a user-type member? */
+   { if (Member->Type->UserType->GetTable()	/* is locally defined? */
+			== Scope->UserTypes)
+	{ PrevMember = Member->Pred();		/* is not defined before? */
+	  while(PrevMember)
+	     { if (Member->Type == PrevMember->Type)
+		  { return(FALSE);	/* already locally defined before */
+		   } /* fi */
+	       PrevMember = PrevMember->Pred();
+	      } /* elihw */
+	  return(TRUE);		/* yes, we do need a local definition here */
+	 } /* fi */
+     else
+	{ return(FALSE);	/* somewhere else defined */
+	 } /* esle */
+    } /* fi */
+else
+   { return(FALSE);	/* not a user-type */
+    } /* esle */
+
+} /* end of SIR_UserType::LocalUTypeNeedsDef */
+
+
+sir_usertypes *SIR_UserType::GetTable(void)	/* determines its table */
+{						/* (NULL if not in a table) */
+
+return((sir_usertypes*) Head());
+
+} /* end of SIR_UserType::GetTable */
+
+
+sir_symbols *SIR_UserType::GetScope(void)	/* determines the scope */
+{						/* (NULL if no links) */
+sir_usertypes	*Table;
+
+if ((Table = GetTable()))
+   { return(Table->Symbols);
+    } /* fi */
+
+return(NULL);
+
+} /* end of SIR_UserType::GetScope */
+
+
 ERROR SIR_UserType::DFS_ForAllNodes(	/* iterator for over all nodes */
 	sir_node_mptr	MemberFct,	/* (depth first) */
 	sir_node_marg	MemberFctArg)
@@ -458,8 +587,8 @@ else
 const char *SIR_UserType::PrettyString(		/* creates a SpecC string */
 	string		*Buffer,		/* (one of the ones below) */
 	BOOL		IncludeNotes,
-	BOOL		CplusplusMode = FALSE,
-	BOOL		UserTypeDefReq = FALSE)
+	BOOL		CplusplusMode /* = FALSE */,
+	BOOL		UserTypeDefReq /* = FALSE */)
 {
 
 assert(Buffer != NULL);
@@ -588,40 +717,8 @@ return(Buffer->chars());
 } /* end of SIR_UserType::PrettyStringD */
 
 
-BOOL SIR_UserType::LocalUTypeNeedsDef(	/* check whether a user-type member */
-	sir_member	*Member)	/* needs a local definition	    */
-{
-sir_member	*PrevMember;
-
-assert(Member != NULL);
-assert(Class != SIR_USERTYPE_ENUM);	/* only applies to struct/union */
-assert(Scope != NULL);			/* union/struct have local scope */
-
-if (Member->Type->UserType)	/* is a user-type member? */
-   { if (Member->Type->UserType->GetTable()	/* is locally defined? */
-			== Scope->UserTypes)
-	{ PrevMember = Member->Pred();		/* is not defined before? */
-	  while(PrevMember)
-	     { if (Member->Type == PrevMember->Type)
-		  { return(FALSE);	/* already locally defined before */
-		   } /* fi */
-	       PrevMember = PrevMember->Pred();
-	      } /* elihw */
-	  return(TRUE);		/* yes, we do need a local definition here */
-	 } /* fi */
-     else
-	{ return(FALSE);	/* somewhere else defined */
-	 } /* esle */
-    } /* fi */
-else
-   { return(FALSE);	/* not a user-type */
-    } /* esle */
-
-} /* end of SIR_UserType::LocalUTypeNeedsDef */
-
-
 ERROR SIR_UserType::WriteSC(	/* (re-) generates SpecC source code */
-	FILE		*File,	/* (for usertype _definitions_) */
+	GL_HANDLE	*IO,	/* (for usertype _definitions_) */
 	BOOL		WriteNotes,
 	BOOL		CplusplusMode)
 {
@@ -631,81 +728,114 @@ string		Buffer;
 assert(IsDefinition());	/* don't handle declarations here */
 
 if (Name)
-   { SIR_LineInfo::WriteNL(File);
-     fputs(ClassName(), File);
-     fputc(' ', File);
-     fputs(Name->chars(), File);
-     fputc(' ', File);
+   { if (  (Member = Members->First())
+	 &&(Member->LineInfo))
+	{ // guess location at location of first member
+	  // (this avoids line directives inside the user type)
+	  if ((SIR_Error = Member->LineInfo->WriteSC(IO, true)))
+	     { return(SIR_Error);
+	      } /* fi */
+	 } /* fi */
+     else
+	{ SIR_LineInfo::WriteNewSC(IO, true);
+	 } /* esle */
+     IO->PutS(ClassName());
+     IO->PutC(' ');
+     IO->PutS(*Name);
+     IO->PutC(' ');
     } /* fi */
 else
    { if (TypeDef)
 	{ assert(TypeDef->StorageClass == SIR_STORAGE_TYPEDEF);
-	  SIR_LineInfo::WriteNL(File);
-	  fputs("typedef ", File);
-	  fputs(ClassName(), File);
-	  fputc(' ', File);
+	  if (  (Member = Members->First())
+	      &&(Member->LineInfo))
+	     { // guess location at location of first member
+	       if ((SIR_Error = Member->LineInfo->WriteSC(IO, true)))
+		  { return(SIR_Error);
+		   } /* fi */
+	      } /* fi */
+	  else
+	     { SIR_LineInfo::WriteNewSC(IO, true);
+	      } /* esle */
+	  IO->PutS("typedef ");
+	  IO->PutS(ClassName());
+	  IO->PutC(' ');
 	 } /* fi */
      else
 	{ if (Class == SIR_USERTYPE_ENUM)
-	     { SIR_LineInfo::WriteNL(File);
-	       fputs(ClassName(), File);
-	       fputc(' ', File);
+	     { if (  (Member = Members->First())
+		   &&(Member->LineInfo))
+		  { // guess location at location of first member
+		    if ((SIR_Error = Member->LineInfo->WriteSC(IO, true)))
+		       { return(SIR_Error);
+			} /* fi */
+		   } /* fi */
+	       else
+		  { SIR_LineInfo::WriteNewSC(IO, true);
+		   } /* esle */
+	       IO->PutS(ClassName());
+	       IO->PutC(' ');
 	      } /* fi */
 	  else
 	     { /* unnamed structs/unions will be defined implicitly! */
-	       return(SIR_ERROR_NO_ERROR);
+	       assert(FALSE);	/* this case is not handled here */
 	      } /* esle */
 	 } /* esle */
     } /* esle */
 
-fputc('{', File);
 if (Class == SIR_USERTYPE_ENUM)
-   { Member = Members->First();
+
+   { IO->TabStepUp();
+     IO->PutC('{');
+     Member = Members->First();
      while(Member)
 	{ assert(Member->Symbol != NULL);	/* enum has always symbols */
 	  if (Member->LineInfo)
-	     { if ((SIR_Error = Member->LineInfo->WriteSC(File, true)))
+	     { if ((SIR_Error = Member->LineInfo->WriteSC(IO, true)))
 		  { return(SIR_Error);
 		   } /* fi */
 	      } /* fi */
 	  else
-	     { SIR_LineInfo::WriteNewSC(File, true);
+	     { SIR_LineInfo::WriteNewSC(IO, true, false, &Member->LineInfo);
 	      } /* esle */
-	  fputs(Member->Symbol->Name.chars(), File);
-	  fprintf(File, "=%d", Member->Symbol->EnumValue);
+	  IO->PutS(Member->Symbol->Name);
+	  IO->PrintF("=%d", Member->Symbol->EnumValue);
 	  Member = Member->Succ();
 	  if (Member)
-	     { fputc(',', File);
+	     { IO->PutC(',');
 	      } /* fi */
 	 } /* elihw */
      if (  (! Name)
 	 &&(TypeDef))
-	{ fputs(" }", File);
+	{ IO->PutS(" }");
 	  if (TypeDef->LineInfo)
-	     { if ((SIR_Error = TypeDef->LineInfo->WriteSC(File, true)))
+	     { if ((SIR_Error = TypeDef->LineInfo->WriteSC(IO, true)))
 		  { return(SIR_Error);
 		   } /* fi */
 	      } /* fi */
 	  else
-	     { SIR_LineInfo::WriteNewSC(File, true);
+	     { SIR_LineInfo::WriteNewSC(IO, true, false, &TypeDef->LineInfo);
 	      } /* esle */
-	  fputs(TypeDef->Name.chars(), File);
-	  fputc(';', File);
+	  IO->PutS(TypeDef->Name);
+	  IO->PutC(';');
+	  IO->TabStepDown();
 	  if (TypeDef->Notes)
-	     { if ((SIR_Error = TypeDef->Notes->WriteSC(File,
+	     { if ((SIR_Error = TypeDef->Notes->WriteSC(IO,
 						WriteNotes, CplusplusMode)))
 		  { return(SIR_Error);
 		   } /* fi */
 	      } /* fi */
 	 } /* fi */
      else
-	{ fputs(" };", File);
+	{ SIR_LineInfo::WriteNL(IO, false, true);	// IO->PutS(" }");
+	  IO->PutC(';');
+	  IO->TabStepDown();
 	 } /* fi */
      Member = Members->First();
      while(Member)
 	{ assert(Member->Symbol != NULL);	/* enum has always symbols */
 	  if (Member->Symbol->Notes)
-	     { if ((SIR_Error = Member->Symbol->Notes->WriteSC(File,
+	     { if ((SIR_Error = Member->Symbol->Notes->WriteSC(IO,
 						WriteNotes, CplusplusMode)))
 		  { return(SIR_Error);
 		   } /* fi */
@@ -713,16 +843,20 @@ if (Class == SIR_USERTYPE_ENUM)
 	  Member = Member->Succ();
 	 } /* elihw */
     } /* fi */
+
 else	/* SIR_USERTYPE_STRUCT or SIR_USERTYPE_UNION */
-   { Member = Members->First();
+
+   { IO->TabStepUp();
+     IO->PutC('{');
+     Member = Members->First();
      while(Member)
 	{ if (Member->LineInfo)
-	     { if ((SIR_Error = Member->LineInfo->WriteSC(File, true)))
+	     { if ((SIR_Error = Member->LineInfo->WriteSC(IO, true)))
 		  { return(SIR_Error);
 		   } /* fi */
 	      } /* fi */
 	  else
-	     { SIR_LineInfo::WriteNewSC(File, true);
+	     { SIR_LineInfo::WriteNewSC(IO, true, false, &Member->LineInfo);
 	      } /* esle */
 	  if (Member->Symbol)
 	     { Buffer = Member->Symbol->Name.chars();
@@ -730,20 +864,19 @@ else	/* SIR_USERTYPE_STRUCT or SIR_USERTYPE_UNION */
 	  else
 	     { Buffer = "";
 	      } /* esle */
-	  fputs(Member->Type->PrettyString(
+	  IO->PutS(Member->Type->PrettyString(
 				&Buffer,
 				WriteNotes,
 				CplusplusMode,
 				LocalUTypeNeedsDef(Member),
-				(Class == SIR_USERTYPE_UNION)),
-					File);
+				(Class == SIR_USERTYPE_UNION)));
 	  if (Member->BitFieldSize != SIR_BITFIELD_SIZE_NONE)
-	     { fprintf(File, ":%u", Member->BitFieldSize);
+	     { IO->PrintF(":%u", Member->BitFieldSize);
 	      } /* fi */
-	  fputc(';', File);
+	  IO->PutC(';');
 	  if (  (Member->Symbol)
 	      &&(Member->Symbol->Notes))
-	     { if ((SIR_Error = Member->Symbol->Notes->WriteSC(File,
+	     { if ((SIR_Error = Member->Symbol->Notes->WriteSC(IO,
 						WriteNotes, CplusplusMode)))
 		  { return(SIR_Error);
 		   } /* fi */
@@ -752,33 +885,34 @@ else	/* SIR_USERTYPE_STRUCT or SIR_USERTYPE_UNION */
 	 } /* elihw */
      if (  (! Name)
 	 &&(TypeDef))
-	{ fputs(" }", File);
+	{ IO->PutS(" }");
 	  if (TypeDef->LineInfo)
-	     { if ((SIR_Error = TypeDef->LineInfo->WriteSC(File, true)))
+	     { if ((SIR_Error = TypeDef->LineInfo->WriteSC(IO, true)))
 		  { return(SIR_Error);
 		   } /* fi */
 	      } /* fi */
 	  else
-	     { SIR_LineInfo::WriteNewSC(File, true);
+	     { SIR_LineInfo::WriteNewSC(IO, true, false, &TypeDef->LineInfo);
 	      } /* esle */
-	  fputs(TypeDef->Name.chars(), File);
-	  fputc(';', File);
+	  IO->PutS(TypeDef->Name);
+	  IO->PutC(';');
+	  IO->TabStepDown();
 	  if (TypeDef->Notes)
-	     { if ((SIR_Error = TypeDef->Notes->WriteSC(File,
+	     { if ((SIR_Error = TypeDef->Notes->WriteSC(IO,
 						WriteNotes, CplusplusMode)))
 		  { return(SIR_Error);
 		   } /* fi */
 	      } /* fi */
 	 } /* fi */
      else
-	{ fputs(" };", File);
+	{ SIR_LineInfo::WriteNL(IO, false, true);	// IO->PutS(" }");
+	  IO->PutC(';');
+	  IO->TabStepDown();
 	 } /* fi */
     } /* esle */
 
-if (ferror(File))
-   { SIR_ErrMsg.form("Writing to file failed%s",
-			GL_SystemErrorMessage());
-     return(SIR_ERROR_WRITING_SPECC_FILE);
+if ((SIR_Error = IO->Check()))
+   { return(SIR_Error);
     } /* fi */
 
 return(SIR_ERROR_NO_ERROR);
@@ -795,6 +929,7 @@ assert(NewNote != NULL);
 NewNote->Symbol = NULL;		/* usertype note */
 NewNote->UserType = this;
 NewNote->Label = NULL;
+NewNote->State = NULL;
 
 if (Notes)
    { if (! Notes->Insert(NewNote))
@@ -1037,13 +1172,21 @@ sir_usertype *SIR_UserTypes::FindLocallyViaTypeDef(	/* find an entry via */
 {
 
 assert(TypeDefName != NULL);
-First();
+
+// bug fix 02/12/03, RD:
+// only unnamed usertypes are considered for this search
+// (which are stored at the end of the list)
+
+Last();
 while(Curr())
-   { if (  (Curr()->TypeDef)
+   { if (Curr()->Name)
+	{ break;
+	 } /* fi */
+     if (  (Curr()->TypeDef)
 	 &&(0 == strcmp(TypeDefName, Curr()->TypeDef->Name.chars())))
 	{ return(Curr());
 	 } /* fi */
-     Next();
+     Previous();
     } /* elihw */
 
 return(NULL);	/* not found */
@@ -1251,7 +1394,24 @@ while(ImpCurr)
 	     { Known = FindLocallyViaTypeDef(ImpCurr->TypeDef->Name.chars());
 	      } /* fi */
 	  else
-	     { Known = NULL;	/* no way to find it */
+	     { sir_symbol	*KnownSymbol;
+
+	       if (  (ImpCurr->Class == SIR_USERTYPE_ENUM)
+		   &&(ImpCurr->Members)
+		   &&(ImpCurr->Members->First())
+		   &&((KnownSymbol = Symbols->Find(
+			ImpCurr->Members->First()->Symbol->Name.chars())))
+		   &&(KnownSymbol->Type->UserType)
+		   &&(KnownSymbol->Type->UserType->IsSameStructure(ImpCurr)))
+		  { // found two identical anonymous enum definitions
+		    // (bug fix 04/08/02, RD)
+		    assert(ImpCurr->Name == NULL);
+		    assert(ImpCurr->TypeDef == NULL);
+		    Known = KnownSymbol->Type->UserType;
+		   } /* fi */
+	       else
+		  { Known = NULL;	/* no way to find it */
+		   } /* esle */
 	      } /* esle */
 	 } /* esle */
      if (Known)
@@ -1266,22 +1426,42 @@ while(ImpCurr)
 	      } /* fi */
 	  if (ImpCurr->IsDefinition())	/* imported a definition? */
 	     { if (Known->IsDefinition())
-		  { SIR_ErrMsg.form("Error while importing:"
+		  { if (Known->IsSameStructure(ImpCurr))
+		       { /* merge the two definitions (RD, 02/15/02) */
+			 ImpCurr->SetAlias(Known); /* set alias recursively */
+			} /* fi */
+		    else
+		       { SIR_ErrMsg.form("Error while importing:"
 							GL_ERROR_MSG_NEWLINE
 					"Redefinition of '%s %s'",
 					Known->ClassName(),
 					Known->NameOrUnnamed());
-		    return(SIR_ERROR_REDEFINITION_OF_USERTYPE_3);
+			 return(SIR_ERROR_REDEFINITION_OF_USERTYPE_3);
+			} /* fi */
 		   } /* fi */
-	       /* make declaration a definition */
-	       Known->Members = ImpCurr->Members;	/* take the members */
-	       ImpCurr->Members = NULL;
-	       Known->Scope = ImpCurr->Scope;		/* take the scope */
-	       ImpCurr->Scope = NULL;
-	       Known->Imported = ImpCurr->Imported;	/* update import info */
+	       else
+		  { /* make declaration a definition */
+		    Known->Members = ImpCurr->Members;	/* take the members */
+		    ImpCurr->Members = NULL;
+		    Known->Scope = ImpCurr->Scope;	/* take the scope */
+		    ImpCurr->Scope = NULL;
+		    Known->Imported = ImpCurr->Imported;/* update import info */
+		   } /* esle */
 	      } /* fi */
 	  ImpCurr->Alias = Known;		/* put alias pointer */
+// bug fix 02/15/02, RD:
+// the following is not sufficient because it only updates the plain type
+// (the one without 'const' and/or 'volatile' flags);
+// instead, all type entries pointing to this user-type must be adjusted
+// by unalias'ing the imported type table (see SIR_Design::Integrate())
+// followed by type table integration (see SIR_Types::Integrate());
+// anyways, the following cannot hurt so we'll leave it here
 	  ImpCurr->Type->Alias = Known->Type;	/* (for type too!) */
+
+	  /* also, merge the annotations (bug fix 06/20/02, RD) */
+	  if ((SIR_Error = SIR_Notes::Merge(&Known->Notes, &ImpCurr->Notes)))
+	     { return(SIR_Error);
+	      } /* fi */
 	 } /* fi */
      else	/* not known */
 	{ Insert(Imported->Remove(ImpCurr));	/* just take the imported one */
@@ -1294,33 +1474,141 @@ return(SIR_ERROR_NO_ERROR);
 } /* end of SIR_UserTypes::Integrate */
 
 
+void SIR_UserTypes::NameAnonymousTypes(	/* preprocessing step for WriteSC */
+	void)
+{
+sir_usertype	*UserType,
+		*Succ;
+unsigned int	NumUsers;
+
+// note: this is a preprocessing step before code generation;
+// we need to assign dummy names to anonymous user-types
+// that are used as type for multiple symbols because we
+// don't declare symbols in a declaration list which would
+// be the only option then (bug fix 04/17/02, RD)
+//
+// Example:
+//	enum { e1, e2 } E1, E2;
+// becomes
+//	enum _scc_anonymous_0 { e1, e2 };
+//	enum _scc_anonymous_0 E1;
+//	enum _scc_anonymous_0 E2;
+
+UserType = First();
+while(UserType)
+   { Succ = UserType->Succ();
+     if (  (! UserType->IsNamed())
+	 &&(! UserType->TypeDef))
+	{ NumUsers = Symbols->NumTypeUsers(UserType->Type);
+	  if (NumUsers > 1)		// used by more than one symbol?
+	     { Remove(UserType);		// take out of this table
+	       assert(UserType->Name == NULL);
+	       UserType->Name = new string(	// assign a new name
+				CreateNewName(SIR_CXX_ANONYMOUS_NAME));
+	       Insert(UserType);		// re-insert into this table
+	      } /* fi */
+	 } /* fi */
+     UserType = Succ;
+    } /* elihw */
+
+} /* end of SIR_UserTypes::NameAnonymousTypes */
+
+
+const char *SIR_UserTypes::CreateNewName(	/* generate a new name */
+	const char	*Format)		/* (must contain one %d) */
+{
+static string	NewName;
+int		i;
+
+i = 0;
+do { NewName.form(Format, i++);
+    } while(FindLocally(NewName.chars()));
+
+return(NewName.chars());
+
+} /* end of SIR_UserTypes::CreateNewName */
+
+
+#ifndef NDEBUG
+
+const char *SIR_UserTypes::Print(	/* print SpecC code */
+	void)
+{
+GL_HANDLE	*IO;
+static string	PrintBuffer;
+BOOL		Dummy = FALSE;
+
+if (!(IO = GL_IO::OpenString(&PrintBuffer, GL_IO_WRITE)))
+   { SIR_Error = GL_Error;
+     return(NULL);
+    } /* fi */
+if ((SIR_Error = WriteSC(IO, TRUE, FALSE, &Dummy)))
+   { IO->Close();
+     return(NULL);
+    } /* fi */
+if ((SIR_Error = IO->CloseWithCheck()))
+   { return(NULL);
+    } /* fi */
+
+return(PrintBuffer.chars());
+
+} /* end of SIR_UserTypes::Print */
+
+#endif /* NDEBUG */
+
+
 ERROR SIR_UserTypes::WriteSC(	/* (re-) generates SpecC (or C++) source code */
-	FILE		*File,
+	GL_HANDLE	*IO,
 	BOOL		WriteNotes,
-	BOOL		CplusplusMode)
+	BOOL		CplusplusMode,
+	BOOL		*PutSeparator)
 {
 sir_usertype	*UserType;
 string		Buffer;
+bool		PutIntro,
+		PutIntroInit,
+		PutComments;
 
-/* (1) Named user-defined type declarations */
+/* (0) preprocessing step: (bug fix 04/17/02, RD) */
+
+NameAnonymousTypes();
+
+
+if (  (Symbols->ScopeInfo == SIR_SCOPE_GLOBAL)
+    ||(Symbols->ScopeInfo == SIR_SCOPE_CLASS))
+   { PutIntroInit = true;	/* separate code blocks */
+    } /* fi */
+else
+   { PutIntroInit = false;	/* combine code blocks */
+    } /* esle */
 #ifdef SIR_PUT_COMMENTS_IN_SOURCE
-SIR_LineInfo::WriteNL(File);
-fputs("// (1) named user-defined type declarations", File);
-SIR_LineInfo::WriteNL(File);
+if (Symbols->ScopeInfo == SIR_SCOPE_GLOBAL)
+   { PutComments = true;
+    } /* fi */
+else
+   { PutComments = false;
+    } /* esle */
+#else /* !SIR_PUT_COMMENTS_IN_SOURCE */
+PutComments = false;
 #endif /* SIR_PUT_COMMENTS_IN_SOURCE */
 
+/* (1) Named user-defined type declarations */
+
+PutIntro = PutIntroInit;
 UserType = First();
 while(UserType)
    { if (  (  (  (CplusplusMode)	/* no enum declarations in C++ */
 	       &&(UserType->Class != SIR_USERTYPE_ENUM))
 	    ||(! CplusplusMode))
 	 &&(UserType->IsNamed()))
-	{ SIR_LineInfo::WriteNL(File);
-	  fputs(UserType->PrettyString(&Buffer, WriteNotes, CplusplusMode),
-			File);
-	  fputc(';', File);
+	{ SIR_Symbols::WriteIntro(IO, false,
+			&PutIntro, PutSeparator, PutComments,
+			"named user-defined type declarations");
+	  SIR_LineInfo::WriteNL(IO);
+	  IO->PutS(UserType->PrettyString(&Buffer, WriteNotes, CplusplusMode));
+	  IO->PutC(';');
 	  if (UserType->Notes)
-	     { if ((SIR_Error = UserType->Notes->WriteSC(File, WriteNotes,
+	     { if ((SIR_Error = UserType->Notes->WriteSC(IO, WriteNotes,
 						CplusplusMode)))
 		  { return(SIR_Error);
 		   } /* fi */
@@ -1330,23 +1618,22 @@ while(UserType)
     } /* elihw */
 
 /* (2) Named enumerator definitions */
-#ifdef SIR_PUT_COMMENTS_IN_SOURCE
-SIR_LineInfo::WriteNL(File);
-fputs("// (2) named enumerator definitions", File);
-SIR_LineInfo::WriteNL(File);
-#endif /* SIR_PUT_COMMENTS_IN_SOURCE */
 
+PutIntro = PutIntroInit;
 UserType = First();
 while(UserType)
    { if (  (UserType->Class == SIR_USERTYPE_ENUM)
 	 &&(UserType->IsNamed())
 	 &&(UserType->IsDefinition()))
-	{ if ((SIR_Error = UserType->WriteSC(File, WriteNotes, CplusplusMode)))
+	{ SIR_Symbols::WriteIntro(IO, true,
+			&PutIntro, PutSeparator, PutComments,
+			"named enumerator type definitions");
+	  if ((SIR_Error = UserType->WriteSC(IO, WriteNotes, CplusplusMode)))
 	     { return(SIR_Error);
 	      } /* fi */
 	  if (  (CplusplusMode)
 	      &&(UserType->Notes))
-	     { if ((SIR_Error = UserType->Notes->WriteSC(File, WriteNotes,
+	     { if ((SIR_Error = UserType->Notes->WriteSC(IO, WriteNotes,
 						CplusplusMode)))
 		  { return(SIR_Error);
 		   } /* fi */
@@ -1356,18 +1643,23 @@ while(UserType)
     } /* elihw */
 
 /* (3) Unnamed enumerator definitions (with and without typedefs) */
-#ifdef SIR_PUT_COMMENTS_IN_SOURCE
-SIR_LineInfo::WriteNL(File);
-fputs("// (3) unnamed enumerator definitions", File);
-SIR_LineInfo::WriteNL(File);
-#endif /* SIR_PUT_COMMENTS_IN_SOURCE */
 
+PutIntro = PutIntroInit;
 UserType = First();
 while(UserType)
    { if (  (UserType->Class == SIR_USERTYPE_ENUM)
 	 &&(! UserType->IsNamed()))
 	{ assert(UserType->IsDefinition());
-	  if ((SIR_Error = UserType->WriteSC(File, WriteNotes, CplusplusMode)))
+	  if (  (! UserType->TypeDef)
+	      &&(Symbols->NumTypeUsers(UserType->Type) == 1))
+	     { // skip the ones used only once (bug fix 04/17/02, RD)
+	       UserType = UserType->Succ();
+	       continue;
+	      } /* fi */
+	  SIR_Symbols::WriteIntro(IO, true,
+			&PutIntro, PutSeparator, PutComments,
+			"unnamed enumerator type definitions");
+	  if ((SIR_Error = UserType->WriteSC(IO, WriteNotes, CplusplusMode)))
 	     { return(SIR_Error);
 	      } /* fi */
 	 } /* fi */
@@ -1375,11 +1667,6 @@ while(UserType)
     } /* elihw */
 
 /* (4) User-defined type definitions (named and unnamed structs and unions) */
-#ifdef SIR_PUT_COMMENTS_IN_SOURCE
-SIR_LineInfo::WriteNL(File);
-fputs("// (4) user-defined type definitions", File);
-SIR_LineInfo::WriteNL(File);
-#endif /* SIR_PUT_COMMENTS_IN_SOURCE */
 
 #ifndef NDEBUG
 UserType = First();
@@ -1389,13 +1676,17 @@ while(UserType)
     } /* elihw */
 #endif /* NDEBUG */
 
+PutIntro = PutIntroInit;
 UserType = First();
 while(UserType)
    { if (  (  (UserType->Class == SIR_USERTYPE_STRUCT)
 	    ||(UserType->Class == SIR_USERTYPE_UNION))
-	 &&(UserType->IsDefinition()))
-	{ if ((SIR_Error = WriteOrderedSC(UserType, File, WriteNotes,
-						CplusplusMode)))
+	 &&(UserType->IsDefinition())
+	 &&(! (  (!UserType->Name)
+	       &&(!UserType->TypeDef))))
+	{ if ((SIR_Error = WriteOrderedSC(UserType, IO, WriteNotes,
+					CplusplusMode, PutSeparator,
+					&PutIntro, PutComments)))
 	     { return(SIR_Error);
 	      } /* fi */
 	 } /* fi */
@@ -1406,7 +1697,9 @@ UserType = First();
 while(UserType)
    { if (  (  (UserType->Class == SIR_USERTYPE_STRUCT)
 	    ||(UserType->Class == SIR_USERTYPE_UNION))
-	 &&(UserType->IsDefinition()))
+	 &&(UserType->IsDefinition())
+	 &&(! (  (!UserType->Name)
+	       &&(!UserType->TypeDef))))
 	{ assert(UserType->Color == SIR_RED);	/* assure work is done */
 	  UserType->Color = SIR_WHITE;
 	 } /* fi */
@@ -1419,10 +1712,13 @@ return(SIR_ERROR_NO_ERROR);
 
 
 ERROR SIR_UserTypes::WriteOrderedSC(	/* writes ordered structures/unions */
-	sir_usertype	*Current,	/* (recursiv) */
-	FILE		*File,
+	sir_usertype	*Current,	/* (recursively) */
+	GL_HANDLE	*IO,
 	BOOL		WriteNotes,
-	BOOL		CplusplusMode)
+	BOOL		CplusplusMode,
+	BOOL		*PutSeparator,
+	BOOL		*PutIntro,
+	BOOL		PutComments)
 {
 sir_usertype	*UserType;
 
@@ -1441,11 +1737,14 @@ UserType = First();
 while(UserType)
    { if (  (  (UserType->Class == SIR_USERTYPE_STRUCT)
 	    ||(UserType->Class == SIR_USERTYPE_UNION))
-	 &&(UserType->IsDefinition()))
+	 &&(UserType->IsDefinition())
+	 &&(! (  (!UserType->Name)
+	       &&(!UserType->TypeDef))))
 	{ if (Current->ContainsNested(UserType))
 	     { if ((SIR_Error = WriteOrderedSC(	/* write dependencies first */
-					UserType, File, WriteNotes,
-					CplusplusMode)))
+					UserType, IO, WriteNotes,
+					CplusplusMode, PutSeparator, PutIntro,
+					PutComments)))
 		  { return(SIR_Error);
 		   } /* fi */
 	      } /* fi */
@@ -1453,10 +1752,14 @@ while(UserType)
      UserType = UserType->Succ();
     } /* elihw */
 
-if ((SIR_Error = Current->WriteSC(File, WriteNotes,	/* now write this one */
+SIR_Symbols::WriteIntro(IO, true,
+		PutIntro, PutSeparator, PutComments,
+		"struct/union type definitions");
+if ((SIR_Error = Current->WriteSC(IO, WriteNotes,	/* now write this one */
 					CplusplusMode)))
    { return(SIR_Error);
     } /* fi */
+
 Current->Color = SIR_RED;	/* work is done for this one */
 
 return(SIR_ERROR_NO_ERROR);
