@@ -3,6 +3,7 @@
 // ============================================================================
 // 
 //
+// 11/21/01 <gerstl>	Cleanup of test case
 // 05/24/01 <gerstl>	Initial version
 //
 // ----------------------------------------------------------------------------
@@ -471,6 +472,7 @@ unsigned long __longlong::div( const __longlong& ll )
 #ifdef DEBUG
 
 #include <stdio.h>
+#include <stdlib.h>
 
 
 #if defined(LLONG_MAX) || defined(LONG_LONG_MAX) || defined(QUAD_MAX)
@@ -509,7 +511,7 @@ void testOp(unsigned long long au, unsigned long long bu,
   _longlong<> x = (_longlong<>)xu;
   _longlong<> y = (_longlong<>)yu;
   _longlong<true>    zu;  
-  _longlong_<> z;
+  _longlong<> z;
     
   // signed
   c = a + b;  z = x + y;
@@ -587,18 +589,35 @@ void testOp(unsigned long long au, unsigned long long bu,
 
 // Test operations with shadow class
 void testOpShadow(unsigned long long au, unsigned long long bu,
-	          _longlong_<true> xu,  _longlong_<true> yu)
+	          _longlong<true> xsu,  _longlong<true> ysu)
 { 
   long long a = (long long)au;
-  long long b = (long long)bu;
-  long long c;
-  unsigned long long cu;
+  
+  union {
+    unsigned long long cu;
+    long long          b;
+  };
+  b = bu;
 
-  _longlong_<> x;       x  = (_longlong<>)xu;
-  _longlong_<> y;       y  = (_longlong<>)yu;
-  _longlong_<true>    zu;  
-  _longlong_<> z;
-    
+  long long c;
+
+  // Make sure, our special shadow _longlong_ can be used in a union!
+  union {
+    _longlong_<>        x;
+    _longlong_<true>    xu;
+  };
+  xu = xsu;
+  union {
+    _longlong_<true>    zu;
+    _longlong_<>        y;
+  };
+  y = ysu;
+  union {
+    _longlong_<>        z;
+    _longlong_<true>    yu;
+  };
+  yu = ysu;
+      
   // signed
   c = a + b;  z = x + y;
   if( c != z.toLLong() ) operror("+", a, b, c, z);
@@ -636,6 +655,7 @@ void testOpShadow(unsigned long long au, unsigned long long bu,
   if( c != z.toLLong() ) operror("-", 0, a, c, z);
   
   // unsigned
+  yu = ysu;
   cu = au + bu;  zu = xu + yu;
   if( cu != zu.toULLong() ) operror("+", au, bu, cu, zu);
   cu = au - bu;  zu = xu - yu;
@@ -674,29 +694,21 @@ void testOpShadow(unsigned long long au, unsigned long long bu,
 
 // Test a mix of shadow and normal classes
 void testOpMix(unsigned long long au, unsigned long long bu,
-	       _longlong<true> xu,    _longlong<true> yu)
+	       _longlong<true> xu,    _longlong<true> ysu)
 { 
   long long a = (long long)au;
-  
-  union {
-    unsigned long long cu;
-    long long          b;
-  };
-  b = bu;
-
+  long long b = (long long)bu;
   long long c;
+  unsigned long long cu;
 
-  _longlong<> x = (_longlong<>)xu;
+  _longlong<> x = xu;
   
-  // Make sure, our special shadow _longlong_ can be used in a union!
-  union {
-    _longlong_<true>    zu;
-    _longlong_<>        y;
-  };
-  y = yu;
+  _longlong_<>     y;	y  = ysu;
+  _longlong_<true> yu;  yu = ysu;
   
-  _longlong_<> z;
-    
+  _longlong<>  z;
+  _longlong<true> zu;
+
   // signed
   c = a + b;  z = x + y;
   if( c != z.toLLong() ) operror("+", a, b, c, z);
@@ -789,7 +801,8 @@ int main(int argc, char** argv)
   for(i = 0, au = 0, xu = 0; i <= LONG_BIT*2; au = (au<<1) + ++i%2, xu = (xu<<1) + i%2)
   {
     for(j = 0, bu = 0, yu = 0; j <= LONG_BIT*2; bu = (bu<<1) + ++j%2, yu = (yu<<1) + j%2) {
-      testOp   (au, bu, xu, yu);
+      testOp(au, bu, xu, yu);
+      testOpShadow(au, bu, xu, yu);
       testOpMix(au, bu, xu, yu);
     }
     
@@ -813,7 +826,8 @@ int main(int argc, char** argv)
   for(i = 0, au = 0, xu = 0; i <= LONG_BIT*2; i++, au = (au<<1) + 1, xu = (xu<<1) + 1)
   {
     for(j = 0, bu = 0, yu = 0; j <= LONG_BIT*2; j++, bu = (bu<<1) + 1, yu = (yu<<1) + 1) {
-      testOp   (au, bu, xu, yu);
+      testOp(au, bu, xu, yu);
+      testOpShadow(au, bu, xu, yu);
       testOpMix(au, bu, xu, yu);
     }
   }
@@ -823,6 +837,8 @@ int main(int argc, char** argv)
   {
     for(j = 0, bu = 0, ysu = 0; j <= LONG_BIT*2; bu = (bu<<1) + ++j%2, ysu = (ysu<<1) + j%2) {
       testOp(au, bu, xsu, ysu);
+      testOpShadow(au, bu, xsu, ysu);
+      testOpMix(au, bu, xsu, ysu);
     }
     
     a = (long long)au;
@@ -846,6 +862,8 @@ int main(int argc, char** argv)
   {
     for(j = 0, bu = 0, ysu = 0; j <= LONG_BIT*2; j++, bu = (bu<<1) + 1, ysu = (ysu<<1) + 1) {
       testOp(au, bu, xsu, ysu);
+      testOpShadow(au, bu, xsu, ysu);
+      testOpMix(au, bu, xsu, ysu);
     }
   }
 
