@@ -4,10 +4,12 @@
 /* Author: Rainer Doemer			first version: 03/26/01 */
 /************************************************************************/
 
-/* last update: 12/10/01 */
+/* last update: 06/15/04 */
 
 /* modifications: (most recent first)
  *
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 04/12/04 RD	moved RunningThread into the thread_base class
  * 12/10/01 RD	introduced explicit return type of thread function
  * 05/16/01 RD	separated common thread code (in thread.[h|cc]) from
  *		platform-specific thread code (in PosixThread.[h|cc])
@@ -53,6 +55,8 @@ thread_base	*ThreadCreator;	/* creator thread (only valid at creation) */
 pthread_t	ThreadHandle;	/* handle for posix thread */
 pthread_cond_t	CondVar;	/* condition variable to wake up this thread */
 
+static thread_base	*RunningThread;	/* the running thread (or NULL) */
+
 
 thread_base(			/* constructor #1 */
 	thread_base	*Creator);
